# ble_settings.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Feb 20 23:24:52 UTC 2025

"""BLE Settings"""

########################################################################################################################
#
# Warning!! This file is auto-generated. Do not modify it manually.
#
########################################################################################################################

from open_gopro import models, parsers
from open_gopro.api.builders import BleSettingFacade as BleSetting
from open_gopro.domain.communicator_interface import BleMessages, GoProBle
from open_gopro.models.constants import SettingId, settings


class BleSettings(BleMessages[BleSetting.BleSettingMessageBase]):
    # pylint: disable=missing-class-docstring, unused-argument
    """The collection of all BLE Settings.

    To be used by a GoProBle delegate to build setting messages.

    Args:
        communicator (GoProBle): Adapter to read / write settings
    """

    def __init__(self, communicator: GoProBle):

        self.video_resolution: BleSetting[settings.VideoResolution] = BleSetting[settings.VideoResolution](
            communicator, SettingId.VIDEO_RESOLUTION, settings.VideoResolution
        )

        """Video Resolution

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-resolution-2)"""

        self.frames_per_second: BleSetting[settings.FramesPerSecond] = BleSetting[settings.FramesPerSecond](
            communicator, SettingId.FRAMES_PER_SECOND, settings.FramesPerSecond
        )

        """Frames Per Second

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#frames-per-second-3)"""

        self.video_timelapse_rate: BleSetting[settings.VideoTimelapseRate] = BleSetting[settings.VideoTimelapseRate](
            communicator, SettingId.VIDEO_TIMELAPSE_RATE, settings.VideoTimelapseRate
        )

        """Video Timelapse Rate

        How frequently to take a video when performing a Video Timelapse

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-timelapse-rate-5)"""

        self.photo_timelapse_rate: BleSetting[settings.PhotoTimelapseRate] = BleSetting[settings.PhotoTimelapseRate](
            communicator, SettingId.PHOTO_TIMELAPSE_RATE, settings.PhotoTimelapseRate
        )

        """Photo Timelapse Rate

        How frequently to take a photo when performing a Photo Timelapse.

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-timelapse-rate-30)"""

        self.nightlapse_rate: BleSetting[settings.NightlapseRate] = BleSetting[settings.NightlapseRate](
            communicator, SettingId.NIGHTLAPSE_RATE, settings.NightlapseRate
        )

        """Nightlapse Rate

        How frequently to take a video or photo when performing a Nightlapse.

		This controls the Video or Photo Nightlapse rate if Setting 128 is set to 21 or 26 respectively.

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#nightlapse-rate-32)"""

        self.webcam_digital_lenses: BleSetting[settings.WebcamDigitalLenses] = BleSetting[settings.WebcamDigitalLenses](
            communicator, SettingId.WEBCAM_DIGITAL_LENSES, settings.WebcamDigitalLenses
        )

        """Webcam Digital Lenses

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#webcam-digital-lenses-43)"""

        self.auto_power_down: BleSetting[settings.AutoPowerDown] = BleSetting[settings.AutoPowerDown](
            communicator, SettingId.AUTO_POWER_DOWN, settings.AutoPowerDown
        )

        """Auto Power Down

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#auto-power-down-59)"""

        self.gps: BleSetting[settings.Gps] = BleSetting[settings.Gps](communicator, SettingId.GPS, settings.Gps)

        """GPS

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#gps-83)"""

        self.lcd_brightness: BleSetting[int] = BleSetting[int](
            communicator, SettingId.LCD_BRIGHTNESS, parsers.IntByteParserBuilder(1)
        )

        """LCD Brightness

        The LCD brightness as a percentage value from 10-100

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#lcd-brightness-88)"""

        self.led: BleSetting[settings.Led] = BleSetting[settings.Led](communicator, SettingId.LED, settings.Led)

        """LED

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#led-91)"""

        self.video_aspect_ratio: BleSetting[settings.VideoAspectRatio] = BleSetting[settings.VideoAspectRatio](
            communicator, SettingId.VIDEO_ASPECT_RATIO, settings.VideoAspectRatio
        )

        """Video Aspect Ratio

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-aspect-ratio-108)"""

        self.video_lens: BleSetting[settings.VideoLens] = BleSetting[settings.VideoLens](
            communicator, SettingId.VIDEO_LENS, settings.VideoLens
        )

        """Video Lens

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-lens-121)"""

        self.photo_lens: BleSetting[settings.PhotoLens] = BleSetting[settings.PhotoLens](
            communicator, SettingId.PHOTO_LENS, settings.PhotoLens
        )

        """Photo Lens

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-lens-122)"""

        self.time_lapse_digital_lenses: BleSetting[settings.TimeLapseDigitalLenses] = BleSetting[
            settings.TimeLapseDigitalLenses
        ](communicator, SettingId.TIME_LAPSE_DIGITAL_LENSES, settings.TimeLapseDigitalLenses)

        """Time Lapse Digital Lenses

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#time-lapse-digital-lenses-123)"""

        self.photo_output: BleSetting[settings.PhotoOutput] = BleSetting[settings.PhotoOutput](
            communicator, SettingId.PHOTO_OUTPUT, settings.PhotoOutput
        )

        """Photo Output

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-output-125)"""

        self.media_format: BleSetting[settings.MediaFormat] = BleSetting[settings.MediaFormat](
            communicator, SettingId.MEDIA_FORMAT, settings.MediaFormat
        )

        """Media Format

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#media-format-128)"""

        self.anti_flicker: BleSetting[settings.Anti_Flicker] = BleSetting[settings.Anti_Flicker](
            communicator, SettingId.ANTI_FLICKER, settings.Anti_Flicker
        )

        """Anti-Flicker

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#anti-flicker-134)"""

        self.hypersmooth: BleSetting[settings.Hypersmooth] = BleSetting[settings.Hypersmooth](
            communicator, SettingId.HYPERSMOOTH, settings.Hypersmooth
        )

        """Hypersmooth

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#hypersmooth-135)"""

        self.video_horizon_leveling: BleSetting[settings.VideoHorizonLeveling] = BleSetting[
            settings.VideoHorizonLeveling
        ](communicator, SettingId.VIDEO_HORIZON_LEVELING, settings.VideoHorizonLeveling)

        """Video Horizon Leveling

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-horizon-leveling-150)"""

        self.photo_horizon_leveling: BleSetting[settings.PhotoHorizonLeveling] = BleSetting[
            settings.PhotoHorizonLeveling
        ](communicator, SettingId.PHOTO_HORIZON_LEVELING, settings.PhotoHorizonLeveling)

        """Photo Horizon Leveling

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-horizon-leveling-151)"""

        self.video_duration: BleSetting[settings.VideoDuration] = BleSetting[settings.VideoDuration](
            communicator, SettingId.VIDEO_DURATION, settings.VideoDuration
        )

        """Video Duration

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-duration-156)"""

        self.multi_shot_duration: BleSetting[settings.MultiShotDuration] = BleSetting[settings.MultiShotDuration](
            communicator, SettingId.MULTI_SHOT_DURATION, settings.MultiShotDuration
        )

        """Multi Shot Duration

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#multi-shot-duration-157)"""

        self.max_lens: BleSetting[settings.MaxLens] = BleSetting[settings.MaxLens](
            communicator, SettingId.MAX_LENS, settings.MaxLens
        )

        """Max Lens

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#max-lens-162)"""

        self.hindsight: BleSetting[settings.Hindsight] = BleSetting[settings.Hindsight](
            communicator, SettingId.HINDSIGHT, settings.Hindsight
        )

        """HindSight

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#hindsight-167)"""

        self.scheduled_capture: BleSetting[models.ScheduledCapture] = BleSetting[models.ScheduledCapture](
            communicator, SettingId.SCHEDULED_CAPTURE, parsers.ScheduledCaptureParser
        )

        """Scheduled Capture

        Configure or disable the scheduled capture functionality to start encoding at a future time.

		This is a bit-masked value with the following bitwise definitions,
		numbered from least significant bit, with the example showing parsed fields from a sample of `0x00000c8b`.

		| Bit(s) | Definition                    | Example |
		| ------ | ----------------------------- | ------- |
		| 0      | Is Scheduled Capture Enabled? | 1       |
		| 1      | Is 24 hour format?            | 1       |
		| 2-8    | Minute                        | 34      |
		| 9-13   | Hour                          | 12      |
		| 14-63  | Reserved                      | 0       |

		Note that when the scheduled capture time occurs, encoding will be started and continue indefinitely.  One
		of the duration settings (156, 157, 172) can be used to set the encoding duration depending on the camera mode.

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#scheduled-capture-168)"""

        self.photo_single_interval: BleSetting[settings.PhotoSingleInterval] = BleSetting[settings.PhotoSingleInterval](
            communicator, SettingId.PHOTO_SINGLE_INTERVAL, settings.PhotoSingleInterval
        )

        """Photo Single Interval

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-single-interval-171)"""

        self.photo_interval_duration: BleSetting[settings.PhotoIntervalDuration] = BleSetting[
            settings.PhotoIntervalDuration
        ](communicator, SettingId.PHOTO_INTERVAL_DURATION, settings.PhotoIntervalDuration)

        """Photo Interval Duration

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-interval-duration-172)"""

        self.video_performance_mode: BleSetting[settings.VideoPerformanceMode] = BleSetting[
            settings.VideoPerformanceMode
        ](communicator, SettingId.VIDEO_PERFORMANCE_MODE, settings.VideoPerformanceMode)

        """Video Performance Mode

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-performance-mode-173)"""

        self.control_mode: BleSetting[settings.ControlMode] = BleSetting[settings.ControlMode](
            communicator, SettingId.CONTROL_MODE, settings.ControlMode
        )

        """Control Mode

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#control-mode-175)"""

        self.easy_mode_speed: BleSetting[settings.EasyModeSpeed] = BleSetting[settings.EasyModeSpeed](
            communicator, SettingId.EASY_MODE_SPEED, settings.EasyModeSpeed
        )

        """Easy Mode Speed

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#easy-mode-speed-176)"""

        self.enable_night_photo: BleSetting[settings.EnableNightPhoto] = BleSetting[settings.EnableNightPhoto](
            communicator, SettingId.ENABLE_NIGHT_PHOTO, settings.EnableNightPhoto
        )

        """Enable Night Photo

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#enable-night-photo-177)"""

        self.wireless_band: BleSetting[settings.WirelessBand] = BleSetting[settings.WirelessBand](
            communicator, SettingId.WIRELESS_BAND, settings.WirelessBand
        )

        """Wireless Band

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#wireless-band-178)"""

        self.star_trails_length: BleSetting[settings.StarTrailsLength] = BleSetting[settings.StarTrailsLength](
            communicator, SettingId.STAR_TRAILS_LENGTH, settings.StarTrailsLength
        )

        """Star Trails Length

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#star-trails-length-179)"""

        self.system_video_mode: BleSetting[settings.SystemVideoMode] = BleSetting[settings.SystemVideoMode](
            communicator, SettingId.SYSTEM_VIDEO_MODE, settings.SystemVideoMode
        )

        """System Video Mode

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#system-video-mode-180)"""

        self.video_bit_rate: BleSetting[settings.VideoBitRate] = BleSetting[settings.VideoBitRate](
            communicator, SettingId.VIDEO_BIT_RATE, settings.VideoBitRate
        )

        """Video Bit Rate

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-bit-rate-182)"""

        self.bit_depth: BleSetting[settings.BitDepth] = BleSetting[settings.BitDepth](
            communicator, SettingId.BIT_DEPTH, settings.BitDepth
        )

        """Bit Depth

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#bit-depth-183)"""

        self.profiles: BleSetting[settings.Profiles] = BleSetting[settings.Profiles](
            communicator, SettingId.PROFILES, settings.Profiles
        )

        """Profiles

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#profiles-184)"""

        self.video_easy_mode: BleSetting[settings.VideoEasyMode] = BleSetting[settings.VideoEasyMode](
            communicator, SettingId.VIDEO_EASY_MODE, settings.VideoEasyMode
        )

        """Video Easy Mode

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-easy-mode-186)"""

        self.lapse_mode: BleSetting[settings.LapseMode] = BleSetting[settings.LapseMode](
            communicator, SettingId.LAPSE_MODE, settings.LapseMode
        )

        """Lapse Mode

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#lapse-mode-187)"""

        self.max_lens_mod: BleSetting[settings.MaxLensMod] = BleSetting[settings.MaxLensMod](
            communicator, SettingId.MAX_LENS_MOD, settings.MaxLensMod
        )

        """Max Lens Mod

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#max-lens-mod-189)"""

        self.max_lens_mod_enable: BleSetting[settings.MaxLensModEnable] = BleSetting[settings.MaxLensModEnable](
            communicator, SettingId.MAX_LENS_MOD_ENABLE, settings.MaxLensModEnable
        )

        """Max Lens Mod Enable

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#max-lens-mod-enable-190)"""

        self.easy_night_photo: BleSetting[settings.EasyNightPhoto] = BleSetting[settings.EasyNightPhoto](
            communicator, SettingId.EASY_NIGHT_PHOTO, settings.EasyNightPhoto
        )

        """Easy Night Photo

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#easy-night-photo-191)"""

        self.multi_shot_aspect_ratio: BleSetting[settings.MultiShotAspectRatio] = BleSetting[
            settings.MultiShotAspectRatio
        ](communicator, SettingId.MULTI_SHOT_ASPECT_RATIO, settings.MultiShotAspectRatio)

        """Multi Shot Aspect Ratio

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#multi-shot-aspect-ratio-192)"""

        self.framing: BleSetting[settings.Framing] = BleSetting[settings.Framing](
            communicator, SettingId.FRAMING, settings.Framing
        )

        """Framing

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#framing-193)"""

        self.camera_volume: BleSetting[settings.CameraVolume] = BleSetting[settings.CameraVolume](
            communicator, SettingId.CAMERA_VOLUME, settings.CameraVolume
        )

        """Camera Volume

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#camera-volume-216)"""

        self.setup_screen_saver: BleSetting[settings.SetupScreenSaver] = BleSetting[settings.SetupScreenSaver](
            communicator, SettingId.SETUP_SCREEN_SAVER, settings.SetupScreenSaver
        )

        """Setup Screen Saver

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#setup-screen-saver-219)"""

        self.setup_language: BleSetting[settings.SetupLanguage] = BleSetting[settings.SetupLanguage](
            communicator, SettingId.SETUP_LANGUAGE, settings.SetupLanguage
        )

        """Setup Language

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#setup-language-223)"""

        self.photo_mode: BleSetting[settings.PhotoMode] = BleSetting[settings.PhotoMode](
            communicator, SettingId.PHOTO_MODE, settings.PhotoMode
        )

        """Photo Mode

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-mode-227)"""

        self.video_framing: BleSetting[settings.VideoFraming] = BleSetting[settings.VideoFraming](
            communicator, SettingId.VIDEO_FRAMING, settings.VideoFraming
        )

        """Video Framing

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-framing-232)"""

        self.multi_shot_framing: BleSetting[settings.MultiShotFraming] = BleSetting[settings.MultiShotFraming](
            communicator, SettingId.MULTI_SHOT_FRAMING, settings.MultiShotFraming
        )

        """Multi Shot Framing

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#multi-shot-framing-233)"""

        self.frame_rate: BleSetting[settings.FrameRate] = BleSetting[settings.FrameRate](
            communicator, SettingId.FRAME_RATE, settings.FrameRate
        )

        """Frame Rate

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#frame-rate-234)"""

        super().__init__(communicator)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Commands related to controlling the camera's operation
"""

import google.protobuf.descriptor

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor



# request_get_preset_status_pb2.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Apr 24 17:26:18 UTC 2025

"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1frequest_get_preset_status.proto\x12\nopen_gopro"\xc8\x01\n\x16RequestGetPresetStatus\x12D\n\x16register_preset_status\x18\x01 \x03(\x0e2$.open_gopro.EnumRegisterPresetStatus\x12F\n\x18unregister_preset_status\x18\x02 \x03(\x0e2$.open_gopro.EnumRegisterPresetStatus\x12 \n\x18use_constant_setting_ids\x18\x03 \x01(\x08*l\n\x18EnumRegisterPresetStatus\x12!\n\x1dREGISTER_PRESET_STATUS_PRESET\x10\x01\x12-\n)REGISTER_PRESET_STATUS_PRESET_GROUP_ARRAY\x10\x02'
)
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "request_get_preset_status_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _ENUMREGISTERPRESETSTATUS._serialized_start = 250
    _ENUMREGISTERPRESETSTATUS._serialized_end = 358
    _REQUESTGETPRESETSTATUS._serialized_start = 48
    _REQUESTGETPRESETSTATUS._serialized_end = 248



# ble_statuses.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Feb 20 23:24:52 UTC 2025

"""BLE Statuses"""

########################################################################################################################
#
# Warning!! This file is auto-generated. Do not modify it manually.
#
########################################################################################################################

from construct import Flag, GreedyString, Int8ub, Int32ub, Int64ub

from open_gopro.api.builders import BleStatusFacade as BleStatus
from open_gopro.domain.communicator_interface import BleMessages, GoProBle
from open_gopro.models.constants import StatusId
from open_gopro.models.constants.statuses import *  # pylint: disable = wildcard-import, unused-wildcard-import


class BleStatuses(BleMessages[BleStatus.BleStatusMessageBase]):
    """All of the BLE Statuses.

    To be used by a GoProBle delegate to build status messages.

    Args:
        communicator (GoProBle): Adapter to read / write settings
    """

    def __init__(self, communicator: GoProBle) -> None:

        self.battery_present: BleStatus[bool] = BleStatus(communicator, StatusId.BATTERY_PRESENT, Flag)
        """Battery Present

        Is the system's internal battery present?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#battery-present-1)"""

        self.internal_battery_bars: BleStatus[InternalBatteryBars] = BleStatus(
            communicator, StatusId.INTERNAL_BATTERY_BARS, InternalBatteryBars
        )
        """Internal Battery Bars

        Rough approximation of internal battery level in bars (or charging)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#internal-battery-bars-2)"""

        self.overheating: BleStatus[bool] = BleStatus(communicator, StatusId.OVERHEATING, Flag)
        """Overheating

        Is the system currently overheating?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#overheating-6)"""

        self.busy: BleStatus[bool] = BleStatus(communicator, StatusId.BUSY, Flag)
        """Busy

        Is the camera busy?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#busy-8)"""

        self.quick_capture: BleStatus[bool] = BleStatus(communicator, StatusId.QUICK_CAPTURE, Flag)
        """Quick Capture

        Is Quick Capture feature enabled?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#quick-capture-9)"""

        self.encoding: BleStatus[bool] = BleStatus(communicator, StatusId.ENCODING, Flag)
        """Encoding

        Is the system currently encoding?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#encoding-10)"""

        self.lcd_lock: BleStatus[bool] = BleStatus(communicator, StatusId.LCD_LOCK, Flag)
        """LCD Lock

        Is LCD lock active?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#lcd-lock-11)"""

        self.video_encoding_duration: BleStatus[int] = BleStatus(
            communicator, StatusId.VIDEO_ENCODING_DURATION, Int32ub
        )
        """Video Encoding Duration

        When encoding video, this is the duration (seconds) of the video so far; 0 otherwise

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#video-encoding-duration-13)"""

        self.wireless_connections_enabled: BleStatus[bool] = BleStatus(
            communicator, StatusId.WIRELESS_CONNECTIONS_ENABLED, Flag
        )
        """Wireless Connections Enabled

        Are Wireless Connections enabled?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wireless-connections-enabled-17)"""

        self.pairing_state: BleStatus[PairingState] = BleStatus(communicator, StatusId.PAIRING_STATE, PairingState)
        """Pairing State

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#pairing-state-19)"""

        self.last_pairing_type: BleStatus[LastPairingType] = BleStatus(
            communicator, StatusId.LAST_PAIRING_TYPE, LastPairingType
        )
        """Last Pairing Type

        The last type of pairing in which the camera was engaged

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#last-pairing-type-20)"""

        self.last_pairing_success: BleStatus[int] = BleStatus(communicator, StatusId.LAST_PAIRING_SUCCESS, Int32ub)
        """Last Pairing Success

        Time since boot (milliseconds) of last successful pairing complete action

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#last-pairing-success-21)"""

        self.wifi_scan_state: BleStatus[WifiScanState] = BleStatus(
            communicator, StatusId.WIFI_SCAN_STATE, WifiScanState
        )
        """Wifi Scan State

        State of current scan for WiFi Access Points

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wifi-scan-state-22)"""

        self.last_wifi_scan_success: BleStatus[int] = BleStatus(communicator, StatusId.LAST_WIFI_SCAN_SUCCESS, Int8ub)
        """Last Wifi Scan Success

        Time since boot (milliseconds) that the WiFi Access Point scan completed

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#last-wifi-scan-success-23)"""

        self.wifi_provisioning_state: BleStatus[WifiProvisioningState] = BleStatus(
            communicator, StatusId.WIFI_PROVISIONING_STATE, WifiProvisioningState
        )
        """Wifi Provisioning State

        WiFi AP provisioning state

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wifi-provisioning-state-24)"""

        self.remote_version: BleStatus[int] = BleStatus(communicator, StatusId.REMOTE_VERSION, Int8ub)
        """Remote Version

        Wireless remote control version

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#remote-version-26)"""

        self.remote_connected: BleStatus[bool] = BleStatus(communicator, StatusId.REMOTE_CONNECTED, Flag)
        """Remote Connected

        Is a wireless remote control connected?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#remote-connected-27)"""

        self.connected_wifi_ssid: BleStatus[str] = BleStatus(
            communicator, StatusId.CONNECTED_WIFI_SSID, GreedyString(encoding="utf-8")
        )
        """Connected WiFi SSID

        The name of the wireless network that the camera is connected to where the camera is acting as a client/station.

		When read via BLE, this value is big-endian byte-encoded int32.

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#connected-wifi-ssid-29)"""

        self.access_point_ssid: BleStatus[str] = BleStatus(
            communicator, StatusId.ACCESS_POINT_SSID, GreedyString(encoding="utf-8")
        )
        """Access Point SSID

        The name of the network that the camera sets up in AP mode for other devices to connect to.

		When read via BLE, this value is big-endian byte-encoded int32.

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#access-point-ssid-30)"""

        self.connected_devices: BleStatus[int] = BleStatus(communicator, StatusId.CONNECTED_DEVICES, Int8ub)
        """Connected Devices

        The number of wireless devices connected to the camera

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#connected-devices-31)"""

        self.preview_stream: BleStatus[bool] = BleStatus(communicator, StatusId.PREVIEW_STREAM, Flag)
        """Preview Stream

        Is Preview Stream enabled?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#preview-stream-32)"""

        self.primary_storage: BleStatus[PrimaryStorage] = BleStatus(
            communicator, StatusId.PRIMARY_STORAGE, PrimaryStorage
        )
        """Primary Storage

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#primary-storage-33)"""

        self.remaining_photos: BleStatus[int] = BleStatus(communicator, StatusId.REMAINING_PHOTOS, Int32ub)
        """Remaining Photos

        How many photos can be taken with current settings before sdcard is full.

		Alternatively, this is:

		- the remaining timelapse capability if Setting 128 is set to Timelapse Photo
		- the remaining nightlapse capability if Setting 128 is set to Nightlapse Photo

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#remaining-photos-34)"""

        self.remaining_video_time: BleStatus[int] = BleStatus(communicator, StatusId.REMAINING_VIDEO_TIME, Int32ub)
        """Remaining Video Time

        How many seconds of video can be captured with current settings before sdcard is full

		Alternatively, this is:

		- the remaining timelapse capability if Setting 128 is set to Timelapse Video
		- the remaining nightlapse capability if Setting 128 is set to Nightlapse Video

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#remaining-video-time-35)"""

        self.photos: BleStatus[int] = BleStatus(communicator, StatusId.PHOTOS, Int32ub)
        """Photos

        Total number of photos on sdcard

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#photos-38)"""

        self.videos: BleStatus[int] = BleStatus(communicator, StatusId.VIDEOS, Int32ub)
        """Videos

        Total number of videos on sdcard

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#videos-39)"""

        self.ota: BleStatus[Ota] = BleStatus(communicator, StatusId.OTA, Ota)
        """OTA

        The current status of Over The Air (OTA) update

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#ota-41)"""

        self.pending_fw_update_cancel: BleStatus[bool] = BleStatus(
            communicator, StatusId.PENDING_FW_UPDATE_CANCEL, Flag
        )
        """Pending FW Update Cancel

        Is there a pending request to cancel a firmware update download?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#pending-fw-update-cancel-42)"""

        self.locate: BleStatus[bool] = BleStatus(communicator, StatusId.LOCATE, Flag)
        """Locate

        Is locate camera feature active?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#locate-45)"""

        self.timelapse_interval_countdown: BleStatus[int] = BleStatus(
            communicator, StatusId.TIMELAPSE_INTERVAL_COUNTDOWN, Int32ub
        )
        """Timelapse Interval Countdown

        The current timelapse interval countdown value (e.g. 5...4...3...2...1...)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#timelapse-interval-countdown-49)"""

        self.sd_card_remaining: BleStatus[int] = BleStatus(communicator, StatusId.SD_CARD_REMAINING, Int64ub)
        """SD Card Remaining

        Remaining space on the sdcard in Kilobytes

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#sd-card-remaining-54)"""

        self.preview_stream_available: BleStatus[bool] = BleStatus(
            communicator, StatusId.PREVIEW_STREAM_AVAILABLE, Flag
        )
        """Preview Stream Available

        Is preview stream supported in current recording/mode/secondary-stream?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#preview-stream-available-55)"""

        self.wifi_bars: BleStatus[int] = BleStatus(communicator, StatusId.WIFI_BARS, Int8ub)
        """Wifi Bars

        WiFi signal strength in bars

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wifi-bars-56)"""

        self.active_hilights: BleStatus[int] = BleStatus(communicator, StatusId.ACTIVE_HILIGHTS, Int8ub)
        """Active Hilights

        The number of hilights in currently-encoding video (value is set to 0 when encoding stops)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#active-hilights-58)"""

        self.time_since_last_hilight: BleStatus[int] = BleStatus(
            communicator, StatusId.TIME_SINCE_LAST_HILIGHT, Int32ub
        )
        """Time Since Last Hilight

        Time since boot (milliseconds) of most recent hilight in encoding video (set to 0 when encoding stops)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#time-since-last-hilight-59)"""

        self.minimum_status_poll_period: BleStatus[int] = BleStatus(
            communicator, StatusId.MINIMUM_STATUS_POLL_PERIOD, Int32ub
        )
        """Minimum Status Poll Period

        The minimum time between camera status updates (milliseconds). Best practice is to not poll for status more
		often than this

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#minimum-status-poll-period-60)"""

        self.liveview_exposure_select_mode: BleStatus[LiveviewExposureSelectMode] = BleStatus(
            communicator, StatusId.LIVEVIEW_EXPOSURE_SELECT_MODE, LiveviewExposureSelectMode
        )
        """Liveview Exposure Select Mode

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#liveview-exposure-select-mode-65)"""

        self.liveview_y: BleStatus[int] = BleStatus(communicator, StatusId.LIVEVIEW_Y, Int8ub)
        """Liveview Y

        Liveview Exposure Select: y-coordinate (percent)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#liveview-y-66)"""

        self.liveview_x: BleStatus[int] = BleStatus(communicator, StatusId.LIVEVIEW_X, Int8ub)
        """Liveview X

        Liveview Exposure Select: y-coordinate (percent)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#liveview-x-67)"""

        self.gps_lock: BleStatus[bool] = BleStatus(communicator, StatusId.GPS_LOCK, Flag)
        """GPS Lock

        Does the camera currently have a GPS lock?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#gps-lock-68)"""

        self.ap_mode: BleStatus[bool] = BleStatus(communicator, StatusId.AP_MODE, Flag)
        """AP Mode

        Is AP mode enabled?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#ap-mode-69)"""

        self.internal_battery_percentage: BleStatus[int] = BleStatus(
            communicator, StatusId.INTERNAL_BATTERY_PERCENTAGE, Int8ub
        )
        """Internal Battery Percentage

        Internal battery level as percentage

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#internal-battery-percentage-70)"""

        self.microphone_accessory: BleStatus[MicrophoneAccessory] = BleStatus(
            communicator, StatusId.MICROPHONE_ACCESSORY, MicrophoneAccessory
        )
        """Microphone Accessory

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#microphone-accessory-74)"""

        self.zoom_level: BleStatus[int] = BleStatus(communicator, StatusId.ZOOM_LEVEL, Int8ub)
        """Zoom Level

        Digital Zoom level as percentage

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#zoom-level-75)"""

        self.wireless_band: BleStatus[WirelessBand] = BleStatus(communicator, StatusId.WIRELESS_BAND, WirelessBand)
        """Wireless Band

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wireless-band-76)"""

        self.zoom_available: BleStatus[bool] = BleStatus(communicator, StatusId.ZOOM_AVAILABLE, Flag)
        """Zoom Available

        Is Digital Zoom feature available?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#zoom-available-77)"""

        self.mobile_friendly: BleStatus[bool] = BleStatus(communicator, StatusId.MOBILE_FRIENDLY, Flag)
        """Mobile Friendly

        Are current video settings mobile friendly? (related to video compression and frame rate)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#mobile-friendly-78)"""

        self.ftu: BleStatus[bool] = BleStatus(communicator, StatusId.FTU, Flag)
        """FTU

        Is the camera currently in First Time Use (FTU) UI flow?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#ftu-79)"""

        self.num_5ghz_available: BleStatus[bool] = BleStatus(communicator, StatusId.NUM_5GHZ_AVAILABLE, Flag)
        """5GHZ Available

        Is 5GHz wireless band available?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#5ghz-available-81)"""

        self.ready: BleStatus[bool] = BleStatus(communicator, StatusId.READY, Flag)
        """Ready

        Is the system fully booted and ready to accept commands?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#ready-82)"""

        self.ota_charged: BleStatus[bool] = BleStatus(communicator, StatusId.OTA_CHARGED, Flag)
        """OTA Charged

        Is the internal battery charged sufficiently to start Over The Air (OTA) update?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#ota-charged-83)"""

        self.cold: BleStatus[bool] = BleStatus(communicator, StatusId.COLD, Flag)
        """Cold

        Is the camera getting too cold to continue recording?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#cold-85)"""

        self.rotation: BleStatus[Rotation] = BleStatus(communicator, StatusId.ROTATION, Rotation)
        """Rotation

        Rotational orientation of the camera

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#rotation-86)"""

        self.zoom_while_encoding: BleStatus[bool] = BleStatus(communicator, StatusId.ZOOM_WHILE_ENCODING, Flag)
        """Zoom while Encoding

        Is this camera model capable of zooming while encoding?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#zoom-while-encoding-88)"""

        self.flatmode: BleStatus[int] = BleStatus(communicator, StatusId.FLATMODE, Int8ub)
        """Flatmode

        Current Flatmode ID

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#flatmode-89)"""

        self.video_preset: BleStatus[int] = BleStatus(communicator, StatusId.VIDEO_PRESET, Int32ub)
        """Video Preset

        Current Video Preset (ID)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#video-preset-93)"""

        self.photo_preset: BleStatus[int] = BleStatus(communicator, StatusId.PHOTO_PRESET, Int32ub)
        """Photo Preset

        Current Photo Preset (ID)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#photo-preset-94)"""

        self.timelapse_preset: BleStatus[int] = BleStatus(communicator, StatusId.TIMELAPSE_PRESET, Int32ub)
        """Timelapse Preset

        Current Time Lapse Preset (ID)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#timelapse-preset-95)"""

        self.preset_group: BleStatus[int] = BleStatus(communicator, StatusId.PRESET_GROUP, Int32ub)
        """Preset Group

        Current Preset Group (ID) (corresponds to ui_mode_groups in settings.json)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#preset-group-96)"""

        self.preset: BleStatus[int] = BleStatus(communicator, StatusId.PRESET, Int32ub)
        """Preset

        Current Preset (ID)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#preset-97)"""

        self.preset_modified: BleStatus[int] = BleStatus(communicator, StatusId.PRESET_MODIFIED, Int32ub)
        """Preset Modified

        The value of this status is set to zero when the client sends a Get Preset Status message to the camera.

		The value of this status is set to a non-zero value when:

		- Preset settings submenu is exited in the camera UI (whether any settings were changed or not)
		- A new preset is created
		- A preset is deleted
		- Preset ordering is changed within a preset group
		- A preset is reset to factory defaults

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#preset-modified-98)"""

        self.remaining_live_bursts: BleStatus[int] = BleStatus(communicator, StatusId.REMAINING_LIVE_BURSTS, Int32ub)
        """Remaining Live Bursts

        The number of Live Bursts can be captured with current settings before sdcard is full

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#remaining-live-bursts-99)"""

        self.live_bursts: BleStatus[int] = BleStatus(communicator, StatusId.LIVE_BURSTS, Int32ub)
        """Live Bursts

        Total number of Live Bursts on sdcard

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#live-bursts-100)"""

        self.capture_delay_active: BleStatus[bool] = BleStatus(communicator, StatusId.CAPTURE_DELAY_ACTIVE, Flag)
        """Capture Delay Active

        Is Capture Delay currently active (i.e. counting down)?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#capture-delay-active-101)"""

        self.media_mod_state: BleStatus[MediaModState] = BleStatus(
            communicator, StatusId.MEDIA_MOD_STATE, MediaModState
        )
        """Media Mod State

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#media-mod-state-102)"""

        self.time_warp_speed: BleStatus[TimeWarpSpeed] = BleStatus(
            communicator, StatusId.TIME_WARP_SPEED, TimeWarpSpeed
        )
        """Time Warp Speed

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#time-warp-speed-103)"""

        self.linux_core: BleStatus[bool] = BleStatus(communicator, StatusId.LINUX_CORE, Flag)
        """Linux Core

        Is the system's Linux core active?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#linux-core-104)"""

        self.lens_type: BleStatus[LensType] = BleStatus(communicator, StatusId.LENS_TYPE, LensType)
        """Lens Type

        Camera lens type (reflects changes to lens settings such as 162, 189, 194, ...)

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#lens-type-105)"""

        self.hindsight: BleStatus[bool] = BleStatus(communicator, StatusId.HINDSIGHT, Flag)
        """Hindsight

        Is Video Hindsight Capture Active?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#hindsight-106)"""

        self.scheduled_capture_preset_id: BleStatus[int] = BleStatus(
            communicator, StatusId.SCHEDULED_CAPTURE_PRESET_ID, Int32ub
        )
        """Scheduled Capture Preset ID

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#scheduled-capture-preset-id-107)"""

        self.scheduled_capture: BleStatus[bool] = BleStatus(communicator, StatusId.SCHEDULED_CAPTURE, Flag)
        """Scheduled Capture

        Is Scheduled Capture set?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#scheduled-capture-108)"""

        self.display_mod_status: BleStatus[DisplayModStatus] = BleStatus(
            communicator, StatusId.DISPLAY_MOD_STATUS, DisplayModStatus
        )
        """Display Mod Status

        Note that this is a bitmasked value.

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#display-mod-status-110)"""

        self.sd_card_write_speed_error: BleStatus[bool] = BleStatus(
            communicator, StatusId.SD_CARD_WRITE_SPEED_ERROR, Flag
        )
        """SD Card Write Speed Error

        Is there an SD Card minimum write speed error?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#sd-card-write-speed-error-111)"""

        self.sd_card_errors: BleStatus[int] = BleStatus(communicator, StatusId.SD_CARD_ERRORS, Int8ub)
        """SD Card Errors

        Number of sdcard write speed errors since device booted

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#sd-card-errors-112)"""

        self.turbo_transfer: BleStatus[bool] = BleStatus(communicator, StatusId.TURBO_TRANSFER, Flag)
        """Turbo Transfer

        Is Turbo Transfer active?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#turbo-transfer-113)"""

        self.camera_control_id: BleStatus[CameraControlId] = BleStatus(
            communicator, StatusId.CAMERA_CONTROL_ID, CameraControlId
        )
        """Camera Control ID

        Camera control status ID

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#camera-control-id-114)"""

        self.usb_connected: BleStatus[bool] = BleStatus(communicator, StatusId.USB_CONNECTED, Flag)
        """USB Connected

        Is the camera connected to a PC via USB?

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#usb-connected-115)"""

        self.usb_controlled: BleStatus[UsbControlled] = BleStatus(communicator, StatusId.USB_CONTROLLED, UsbControlled)
        """USB Controlled

        Camera control over USB state

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#usb-controlled-116)"""

        self.sd_card_capacity: BleStatus[int] = BleStatus(communicator, StatusId.SD_CARD_CAPACITY, Int32ub)
        """SD Card Capacity

        Total SD card capacity in Kilobytes

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#sd-card-capacity-117)"""

        self.photo_interval_capture_count: BleStatus[int] = BleStatus(
            communicator, StatusId.PHOTO_INTERVAL_CAPTURE_COUNT, Int32ub
        )
        """Photo Interval Capture Count

        See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#photo-interval-capture-count-118)"""

        super().__init__(communicator)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Defines the structure of protobuf messages for enabling and disabling Turbo Transfer feature
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions
DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class RequestSetTurboActive(google.protobuf.message.Message):
    """*
    Enable/disable display of "Transferring Media" UI

    Response: @ref ResponseGeneric
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ACTIVE_FIELD_NUMBER: builtins.int
    active: builtins.bool
    "Enable or disable Turbo Transfer feature"

    def __init__(self, *, active: builtins.bool | None = ...) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["active", b"active"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["active", b"active"]) -> None: ...

global___RequestSetTurboActive = RequestSetTurboActive



# response.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon Apr 21 22:24:00 UTC 2025

"""GoPro response parsing"""

import enum
import logging
from abc import ABC, abstractmethod
from collections import defaultdict
from typing import Any, Final, Generic, TypeVar

import requests

from open_gopro.domain.exceptions import ResponseParseError
from open_gopro.domain.parser_interface import GlobalParsers, Parser
from open_gopro.domain.types import CameraState, JsonDict, ResponseType
from open_gopro.models import GoProBlePacketHeader, GoProResp
from open_gopro.models.constants import (
    ActionId,
    CmdId,
    ErrorCode,
    FeatureId,
    GoProUUID,
    QueryCmdId,
    SettingId,
    StatusId,
)
from open_gopro.models.proto import EnumResultGeneric
from open_gopro.network.ble import BleUUID
from open_gopro.parsers.json import LambdaJsonParser

CONT_MASK: Final = 0b10000000
HDR_MASK: Final = 0b01100000
GEN_LEN_MASK: Final = 0b00011111
EXT_13_BYTE0_MASK: Final = 0b00011111

logger = logging.getLogger(__name__)

T = TypeVar("T")


class RespBuilder(Generic[T], ABC):
    """Common Response Builder Interface"""

    class _State(enum.Enum):
        """Describes the state of building the response."""

        INITIALIZED = enum.auto()
        ACCUMULATED = enum.auto()
        PARSED = enum.auto()
        ERROR = enum.auto()

    def __init__(self) -> None:
        self._packet: T
        self._status: ErrorCode = ErrorCode.UNKNOWN
        self._state: RespBuilder._State = RespBuilder._State.INITIALIZED
        self._parser: Parser | None = None

    @abstractmethod
    def build(self) -> GoProResp[T]:
        """Build a response

        Returns:
            GoProResp[T]: built response
        """


class HttpRespBuilder(RespBuilder[JsonDict]):
    """HTTP Response Builder

    This is not intended to be fool proof to use as the user must understand which fields are needed.
    Directors should be created if this needs to be simplified.
    """

    def __init__(self) -> None:
        super().__init__()
        self._endpoint: str
        self._response: JsonDict

    def set_response(self, response: JsonDict) -> None:
        """Store the JSON data. This is mandatory.

        Args:
            response (JsonDict): json data_
        """
        self._response = response

    def set_status(self, status: ErrorCode) -> None:
        """Store the status. This is mandatory.

        Args:
            status (ErrorCode): status of response
        """
        self._status = status

    def set_parser(self, parser: Parser) -> None:
        """Store a parser. This is optional.

        Args:
            parser (Parser): monolithic parser
        """
        self._parser = parser

    def set_endpoint(self, endpoint: str) -> None:
        """Store the endpoint. This is mandatory.

        Args:
            endpoint (str): endpoint of response.
        """
        self._endpoint = endpoint

    def build(self) -> GoProResp:
        """Build the GoPro response from the information accumulated about the HTTP response

        Returns:
            GoProResp: built response
        """
        # Is there a parser for this? Most of them do not have one yet.
        data = self._parser.parse(self._response) if self._parser else self._response
        return GoProResp(
            protocol=GoProResp.Protocol.HTTP,
            status=self._status,
            identifier=self._endpoint,
            data=data,
        )


class RequestsHttpRespBuilderDirector:
    """An abstraction to help simplify using the HTTP Response Builder for requests

    Args:
        response (requests.models.Response): direct response from requests
        parser (Parser | None): parsers to use on the requests response
    """

    def __init__(self, response: requests.models.Response, parser: Parser | None) -> None:

        self.response = response
        self.parser = parser or Parser(json_parser=LambdaJsonParser(lambda data: data))

    def __call__(self) -> GoProResp:
        """Build the response

        Returns:
            GoProResp: built response
        """
        builder = HttpRespBuilder()
        builder.set_endpoint(self.response.url)
        builder.set_status(ErrorCode.SUCCESS if self.response.ok else ErrorCode.ERROR)
        builder.set_parser(self.parser)
        builder.set_response(self.response.json() if self.response.text else {})
        return builder.build()


class BleRespBuilder(RespBuilder[bytearray]):
    """BLE Response Builder

    This is not intended to be fool proof to use as the user must understand which fields are needed.
    Directors should be created if this needs to be simplified.
    """

    def __init__(self) -> None:
        self._bytes_remaining = 0
        self._uuid: BleUUID
        self._identifier: ResponseType
        self._feature_id: FeatureId | None = None
        self._action_id: ActionId | None = None
        super().__init__()

    @property
    def is_response_protobuf(self) -> bool:
        """Is this a protobuf response?

        Returns:
            bool: True if protobuf, False otherwise
        """
        return isinstance(self._identifier, (ActionId, FeatureId))

    @classmethod
    def get_response_identifier(cls, uuid: BleUUID, packet: bytearray) -> ResponseType:
        """Get the identifier based on what is currently known about the packet

        Args:
            uuid (BleUUID): UUID packet was received on
            packet (bytearray): raw bytes contained in packet

        Returns:
            ResponseType: identifier of this response
        """
        # If it's a protobuf command
        identifier = packet[0]
        try:
            FeatureId(identifier)
            return ActionId(packet[1])
        # Otherwise it's a TLV command
        except ValueError:
            if uuid is GoProUUID.CQ_SETTINGS_RESP:
                return SettingId(identifier)
            if uuid is GoProUUID.CQ_QUERY_RESP:
                return QueryCmdId(identifier)
            if uuid in [GoProUUID.CQ_COMMAND_RESP, GoProUUID.CN_NET_MGMT_RESP]:
                return CmdId(identifier)
            return uuid

    def set_parser(self, parser: Parser) -> None:
        """Store a parser. This is optional.

        Args:
            parser (Parser): monolithic parser
        """
        self._parser = parser

    def set_packet(self, packet: bytes) -> None:
        """Store the complete data that comprises the response.

        This is mutually exclusive with accumulate. It is only for responses (such as direct UUID reads) that
        do not follow the packet fragmentation scheme.

        Args:
            packet (bytes): packet to store
        """
        self._packet = bytearray(packet)

    def accumulate(self, data: bytes) -> None:
        """Accumulate BLE byte data.

        This is mutually exclusive with accumulate. It should be used in any case where the response follows
        the packet fragmentation scheme.

        Args:
            data (bytes): byte level BLE data
        """
        buf = bytearray(data)
        if buf[0] & CONT_MASK:
            buf.pop(0)
        else:
            # This is a new packet so start with an empty byte array
            self._packet = bytearray([])
            hdr = GoProBlePacketHeader((buf[0] & HDR_MASK) >> 5)
            if hdr is GoProBlePacketHeader.GENERAL:
                self._bytes_remaining = buf[0] & GEN_LEN_MASK
                buf = buf[1:]
            elif hdr is GoProBlePacketHeader.EXT_13:
                self._bytes_remaining = ((buf[0] & EXT_13_BYTE0_MASK) << 8) + buf[1]
                buf = buf[2:]
            elif hdr is GoProBlePacketHeader.EXT_16:
                self._bytes_remaining = (buf[1] << 8) + buf[2]
                buf = buf[3:]

        # Append payload to buffer and update remaining / complete
        self._packet.extend(buf)
        self._bytes_remaining -= len(buf)

        if self._bytes_remaining < 0:
            logger.error("received too much data. parsing is in unknown state")
        elif self._bytes_remaining == 0:
            self._state = RespBuilder._State.ACCUMULATED

    def set_status(self, status: ErrorCode) -> None:
        """Store the status. This is sometimes optional.

        Args:
            status (ErrorCode): status
        """
        self._status = status

    def set_uuid(self, uuid: BleUUID) -> None:
        """Store the UUID. This is mandatory.

        Args:
            uuid (BleUUID): uuid
        """
        self._uuid = uuid

    @property
    def is_finished_accumulating(self) -> bool:
        """Has the response been completely received?

        Returns:
            bool: True if completely received, False if not
        """
        return self._state is not RespBuilder._State.INITIALIZED

    @property
    def _is_protobuf(self) -> bool:
        """Is this response a protobuf response

        Returns:
            bool: Yes if true, No otherwise
        """
        return isinstance(self._identifier, (ActionId, FeatureId))

    @property
    def _is_direct_read(self) -> bool:
        """Is this response a direct read of a BLE characteristic

        Returns:
            bool: Yes if true, No otherwise
        """
        return isinstance(self._identifier, BleUUID)

    def build(self) -> GoProResp:
        """Parse the accumulated response (either from a BLE bytestream or an HTTP JSON dict).

        Raises:
            NotImplementedError: Parsing for this id is not yet supported
            ResponseParseError: Error when parsing data

        Returns:
            GoProResp: built response
        """
        try:
            self._identifier = self.get_response_identifier(self._uuid, self._packet)
            buf = self._packet

            if not self._is_direct_read:  # length byte
                buf.pop(0)
            if self._is_protobuf:  # feature ID byte
                buf.pop(0)

            parsed: Any = None
            query_type: type[StatusId] | type[SettingId] | StatusId | SettingId | None = None
            # Need to delineate QueryCmd responses between settings and status
            if not self._is_protobuf:
                if isinstance(self._identifier, (SettingId, StatusId)):
                    query_type = self._identifier
                elif isinstance(self._identifier, QueryCmdId):
                    if self._identifier in [
                        QueryCmdId.GET_STATUS_VAL,
                        QueryCmdId.REG_STATUS_VAL_UPDATE,
                        QueryCmdId.UNREG_STATUS_VAL_UPDATE,
                        QueryCmdId.STATUS_VAL_PUSH,
                    ]:
                        query_type = StatusId
                    elif self._identifier is QueryCmdId.GET_SETTING_NAME:
                        raise NotImplementedError
                    else:
                        query_type = SettingId

            # Query (setting get value, status get value, etc.)
            if query_type:
                camera_state: CameraState = defaultdict(list)
                self._status = ErrorCode(buf[0])
                buf = buf[1:]
                # Parse all parameters
                while len(buf) != 0:
                    param_len = buf[1]
                    try:
                        param_id = query_type(buf[0])  # type: ignore
                    except ValueError:
                        # We don't handle this entity. Ensure to advance past the value.
                        buf = buf[2 + param_len :]
                        continue
                    buf = buf[2:]
                    # Special case where we register for a push notification for something that does not yet have a value
                    if param_len == 0:
                        camera_state[param_id] = []
                        continue
                    param_val = buf[:param_len]
                    buf = buf[param_len:]

                    # Add parsed value to response's data dict
                    try:
                        if not (parser := GlobalParsers.get_parser(param_id)):
                            # We don't have defined params for all ID's yet. Just store raw bytes
                            logger.warning(f"No parser defined for {param_id}")
                            camera_state[param_id] = param_val.hex(":")
                            continue
                        # These can be more than 1 value so use a list
                        if self._identifier in [
                            QueryCmdId.GET_CAPABILITIES_VAL,
                            QueryCmdId.REG_CAPABILITIES_UPDATE,
                            QueryCmdId.SETTING_CAPABILITY_PUSH,
                        ]:
                            # Parse using parser from global map and append
                            camera_state[param_id].append(parser.parse(param_val))
                        else:
                            # Parse using parser from map and set
                            camera_state[param_id] = parser.parse(param_val)
                    except ValueError:
                        # This is the case where we receive a value that is not defined in our params.
                        # This shouldn't happen and means the documentation needs to be updated. However, it
                        # isn't functionally critical
                        logger.warning(f"{param_id} does not contain a value {param_val}")
                        camera_state[param_id] = param_val
                parsed = camera_state

            else:  # Commands,  Protobuf, and direct Reads
                if is_cmd := isinstance(self._identifier, CmdId):
                    # All (non-protobuf) commands have a status
                    self._status = ErrorCode(buf[0])
                    buf = buf[1:]
                # Use parser if explicitly passed otherwise get global parser
                if not (parser := self._parser or GlobalParsers.get_parser(self._identifier)) and not is_cmd:
                    error_msg = f"No parser exists for {self._identifier}"
                    logger.error(error_msg)
                    raise ResponseParseError(str(self._identifier), self._packet, msg=error_msg)
                # Parse payload if a parser was found.
                if parser:
                    parsed = parser.parse(buf)

                # TODO make status checking an abstract method of a shared base class
                # Attempt to determine and / or extract status (we already got command status above)
                if self._is_direct_read and len(self._packet):
                    # Assume success on direct reads if there was any data
                    self._status = ErrorCode.SUCCESS
                # Check for result field in protobuf's
                elif self._is_protobuf and "result" in parsed:
                    self._status = (
                        ErrorCode.SUCCESS
                        if parsed.get("result") == EnumResultGeneric.RESULT_SUCCESS
                        else ErrorCode.ERROR
                    )
        except Exception as e:
            self._state = RespBuilder._State.ERROR
            raise ResponseParseError(str(self._identifier), buf) from e

        # Recursively scrub away parsing artifacts
        self._state = RespBuilder._State.PARSED

        return GoProResp(protocol=GoProResp.Protocol.BLE, status=self._status, data=parsed, identifier=self._identifier)



# general.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon Apr 21 22:24:00 UTC 2025

"""Monolithic Parser implementations"""

from typing import Any

from construct import BitsInteger, BitStruct, Flag, Int8ub, Int32ub, Padding

from open_gopro.domain.parser_interface import BytesParserBuilder
from open_gopro.models import ScheduledCapture
from open_gopro.parsers.bytes import ConstructDataclassByteParserBuilder

ScheduledCaptureParser = ConstructDataclassByteParserBuilder(
    construct=BitStruct(
        Padding(19),
        "hour" / BitsInteger(5),
        "minute" / BitsInteger(6),
        "is_24_hour" / Flag,
        "is_enabled" / Flag,
    ),
    data_class=ScheduledCapture,
    int_builder=Int32ub,
)


class IntByteParserBuilder(BytesParserBuilder[int]):
    """Built / parse integers to / from bytes

    Args:
        length (int): length of byte array to store integer

    Raises:
        ValueError: _description_
    """

    def __init__(self, length: int) -> None:
        match length:
            case 1:
                self._container = Int8ub
            case 4:
                self._container = Int32ub
            case _:
                raise ValueError(f"Length {length} is not handled")

    def parse(self, data: bytes) -> int:  # noqa: D102
        return self._container.parse(data)

    def build(self, obj: Any) -> bytes:  # noqa: D102
        match obj:
            case int():
                return self._container.build(obj)
            case str():
                return self._container.build(int(obj))
            case _:
                raise TypeError(f"Can not build bytes from object of type {type(obj)}")



# response_generic_pb2.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Apr 24 17:26:18 UTC 2025

"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x16response_generic.proto\x12\nopen_gopro"@\n\x0fResponseGeneric\x12-\n\x06result\x18\x01 \x02(\x0e2\x1d.open_gopro.EnumResultGeneric"%\n\x05Media\x12\x0e\n\x06folder\x18\x01 \x01(\t\x12\x0c\n\x04file\x18\x02 \x01(\t*\xcf\x01\n\x11EnumResultGeneric\x12\x12\n\x0eRESULT_UNKNOWN\x10\x00\x12\x12\n\x0eRESULT_SUCCESS\x10\x01\x12\x15\n\x11RESULT_ILL_FORMED\x10\x02\x12\x18\n\x14RESULT_NOT_SUPPORTED\x10\x03\x12!\n\x1dRESULT_ARGUMENT_OUT_OF_BOUNDS\x10\x04\x12\x1b\n\x17RESULT_ARGUMENT_INVALID\x10\x05\x12!\n\x1dRESULT_RESOURCE_NOT_AVAILABLE\x10\x06'
)
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "response_generic_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _ENUMRESULTGENERIC._serialized_start = 144
    _ENUMRESULTGENERIC._serialized_end = 351
    _RESPONSEGENERIC._serialized_start = 38
    _RESPONSEGENERIC._serialized_end = 102
    _MEDIA._serialized_start = 104
    _MEDIA._serialized_end = 141



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Defines the structure of protobuf message containing generic response to a command
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions
DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _EnumResultGeneric:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumResultGenericEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumResultGeneric.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    RESULT_UNKNOWN: _EnumResultGeneric.ValueType
    RESULT_SUCCESS: _EnumResultGeneric.ValueType
    RESULT_ILL_FORMED: _EnumResultGeneric.ValueType
    RESULT_NOT_SUPPORTED: _EnumResultGeneric.ValueType
    RESULT_ARGUMENT_OUT_OF_BOUNDS: _EnumResultGeneric.ValueType
    RESULT_ARGUMENT_INVALID: _EnumResultGeneric.ValueType
    RESULT_RESOURCE_NOT_AVAILABLE: _EnumResultGeneric.ValueType

class EnumResultGeneric(_EnumResultGeneric, metaclass=_EnumResultGenericEnumTypeWrapper): ...

RESULT_UNKNOWN: EnumResultGeneric.ValueType
RESULT_SUCCESS: EnumResultGeneric.ValueType
RESULT_ILL_FORMED: EnumResultGeneric.ValueType
RESULT_NOT_SUPPORTED: EnumResultGeneric.ValueType
RESULT_ARGUMENT_OUT_OF_BOUNDS: EnumResultGeneric.ValueType
RESULT_ARGUMENT_INVALID: EnumResultGeneric.ValueType
RESULT_RESOURCE_NOT_AVAILABLE: EnumResultGeneric.ValueType
global___EnumResultGeneric = EnumResultGeneric

@typing_extensions.final
class ResponseGeneric(google.protobuf.message.Message):
    """
    Generic Response used across many response / notification messages
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    RESULT_FIELD_NUMBER: builtins.int
    result: global___EnumResultGeneric.ValueType
    "Generic pass/fail/error info"

    def __init__(self, *, result: global___EnumResultGeneric.ValueType | None = ...) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["result", b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___ResponseGeneric = ResponseGeneric

@typing_extensions.final
class Media(google.protobuf.message.Message):
    """*
    A common model to represent a media file
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FOLDER_FIELD_NUMBER: builtins.int
    FILE_FIELD_NUMBER: builtins.int
    folder: builtins.str
    "Directory in which the media is contained"
    file: builtins.str
    "Filename of media"

    def __init__(self, *, folder: builtins.str | None = ..., file: builtins.str | None = ...) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["file", b"file", "folder", b"folder"]
    ) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["file", b"file", "folder", b"folder"]) -> None: ...

global___Media = Media



# util.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Feb 23 22:43:52 UTC 2023

"""Common GUI utilities"""

from __future__ import annotations

import logging
import queue
import threading
from typing import Any, Callable

import cv2

logger = logging.getLogger(__name__)


def display_video_blocking(source: str, printer: Callable = print) -> None:
    """Open a video source to display it, and block until the user stops it by sending 'q'

    Args:
        source (str): video source to display
        printer (Callable): used to display output message. Defaults to print.
    """
    BufferlessVideoCapture(source, printer)


class BufferlessVideoCapture(threading.Thread):
    """Buffer-less video capture to only display the most recent frame

    Open a video source to display it, and block until the user stops it by sending 'q'

    Args:
        source (str): video source to display
        printer (Callable): used to display output message. Defaults to print.
    """

    def __init__(self, source: str, printer: Callable = print) -> None:
        self.printer = printer
        self.printer("Starting viewer...")
        self.cap = cv2.VideoCapture(source + "?overrun_nonfatal=1&fifo_size=50000000", cv2.CAP_FFMPEG)
        self.q: queue.Queue[Any] = queue.Queue()
        super().__init__(daemon=True)
        self.start()
        self.printer("Viewer started")
        self.printer("Press 'q' in viewer to quit")
        while True:
            try:
                frame = self.q.get()
                cv2.imshow("frame", frame)
                if cv2.waitKey(1) & 0xFF == ord("q"):
                    break
            except Exception as e:  # pylint: disable=broad-exception-caught
                logger.warning(e)
                break
        self.cap.release()
        cv2.destroyAllWindows()

    def run(self) -> None:
        """Read frames as soon as they are available, keeping only most recent one"""
        while True:
            try:
                ret, frame = self.cap.read()
                if not ret:
                    self.printer("Received empty frame.")
                    continue
                if not self.q.empty():
                    self.q.get_nowait()  # discard previous (unprocessed) frame
                self.q.put(frame)
            except queue.Empty:
                pass
            except:  # pylint: disable=bare-except
                break



# requests_session.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon May 12 23:03:50 UTC 2025

"""Helper methods for modifying SSL / requests configuration."""

import ssl
from pathlib import Path
from typing import Any

import requests
from requests.adapters import (
    DEFAULT_POOL_TIMEOUT,
    DEFAULT_POOLBLOCK,
    DEFAULT_POOLSIZE,
    DEFAULT_RETRIES,
    HTTPAdapter,
)


# In Python >= 3.13 , ssl.create_default_context() sets ssl.VERIFY_X509_PARTIAL_CHAIN and ssl.VERIFY_X509_STRICT as
# default flags. We need to remove these flags to allow GoPro cameras to work with the requests library.
def create_less_strict_requests_session(cert: Path) -> requests.Session:
    """Create a requests session with less strict SSL verification for GoPro cameras

    Args:
        cert(Path): Path to the CA certificate file to use for SSL verification

    Returns:
        requests.Session: A requests session with a custom SSL context
    """

    class CustomSSLAdapter(HTTPAdapter):
        """HTTPAdapter that allows customizing the SSL context"""

        def __init__(self, ssl_context: ssl.SSLContext, **kwargs: Any) -> None:
            self.ssl_context = ssl_context
            super().__init__(**kwargs)

        def init_poolmanager(self, *args: Any, **kwargs: Any) -> Any:
            """Initialize the pool manager with our custom SSL context"""
            if self.ssl_context:
                kwargs["ssl_context"] = self.ssl_context
            return super().init_poolmanager(*args, **kwargs)

    # Create a custom SSL context with the strict flags removed
    context = ssl.create_default_context(cafile=cert)

    # Remove the new strict verification flags
    context.verify_flags &= ~ssl.VERIFY_X509_PARTIAL_CHAIN
    context.verify_flags &= ~ssl.VERIFY_X509_STRICT

    # Create a session and mount our custom adapter
    session = requests.Session()
    adapter = CustomSSLAdapter(ssl_context=context)
    session.mount("https://", adapter)

    return session



# photo.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Wed, Sep  1, 2021  5:05:45 PM

"""Entrypoint for taking a picture demo."""

import argparse
import asyncio
from pathlib import Path

from rich.console import Console

from open_gopro import WiredGoPro, WirelessGoPro
from open_gopro.gopro_base import GoProBase
from open_gopro.models import constants, proto
from open_gopro.util import add_cli_args_and_parse
from open_gopro.util.logger import setup_logging

console = Console()


async def main(args: argparse.Namespace) -> None:
    logger = setup_logging(__name__, args.log)

    gopro: GoProBase | None = None

    try:
        async with (
            WiredGoPro(args.identifier)
            if args.wired
            else WirelessGoPro(args.identifier, host_wifi_interface=args.wifi_interface)
        ) as gopro:
            assert gopro
            assert (await gopro.http_command.load_preset_group(group=proto.EnumPresetGroup.PRESET_GROUP_ID_PHOTO)).ok

            # Get the media list before
            media_set_before = set((await gopro.http_command.get_media_list()).data.files)

            # Take a photo
            assert (await gopro.http_command.set_shutter(shutter=constants.Toggle.ENABLE)).ok

            # Get the media list after
            media_set_after = set((await gopro.http_command.get_media_list()).data.files)
            # The video (is most likely) the difference between the two sets
            photo = media_set_after.difference(media_set_before).pop()

            # Download the photo
            console.print(f"Downloading {photo.filename}...")
            await gopro.http_command.download_file(camera_file=photo.filename, local_file=args.output)
            console.print(f"Success!! :smiley: File has been downloaded to {Path(args.output).absolute()}")

    except Exception as e:  # pylint: disable = broad-except
        logger.error(repr(e))

    if gopro:
        await gopro.close()


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description="Connect to a GoPro camera, take a photo, then download it.")
    parser.add_argument(
        "--output",
        type=Path,
        help="Where to write the photo to. If not set, write to 'photo.jpg'",
        default=Path("photo.jpg"),
    )
    parser.add_argument(
        "--wired",
        action="store_true",
        help="Set to use wired (USB) instead of wireless (BLE / WIFI) interface",
    )

    return add_cli_args_and_parse(parser)


# Needed for poetry scripts defined in pyproject.toml
def entrypoint() -> None:
    asyncio.run(main(parse_arguments()))


if __name__ == "__main__":
    entrypoint()



# constants.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Wed, Sep  1, 2021  5:05:44 PM

"""Constant numbers shared across the GoPro module. These do not change across Open GoPro Versions"""

# pylint: disable=missing-class-docstring, no-member

from __future__ import annotations

from open_gopro.domain.enum import GoProEnum, GoProIntEnum


class ErrorCode(GoProIntEnum):
    """Status Codes."""

    SUCCESS = 0
    ERROR = 1
    INVALID_PARAM = 2
    UNKNOWN = -1


class CmdId(GoProIntEnum):
    """Command ID's that are written to GoProUUID.CQ_COMMAND."""

    SET_SHUTTER = 0x01
    POWER_DOWN = 0x04
    SLEEP = 0x05
    SET_PAIRING_COMPLETE = 0x03
    SET_DATE_TIME = 0x0D
    GET_DATE_TIME = 0x0E
    SET_DATE_TIME_DST = 0x0F
    GET_DATE_TIME_DST = 0x10
    REBOOT = 0x11
    GET_CAMERA_SETTINGS = 0x12
    GET_CAMERA_STATUSES = 0x13
    GET_CAMERA_CAPABILITIES = 0x32
    SET_WIFI = 0x17
    TAG_HILIGHT = 0x18
    GET_SETTINGS_JSON = 0x3B
    GET_HW_INFO = 0x3C
    LOAD_PRESET_GROUP = 0x3E
    LOAD_PRESET = 0x40
    SET_THIRD_PARTY_CLIENT_INFO = 0x50
    GET_THIRD_PARTY_API_VERSION = 0x51
    REGISTER_ALL_SETTINGS = 0x52
    REGISTER_ALL_STATUSES = 0x53
    UNREGISTER_ALL_SETTINGS = 0x72
    UNREGISTER_ALL_STATUSES = 0x73
    REGISTER_ALL_CAPABILITIES = 0x62
    UNREGISTER_ALL_CAPABILITIES = 0x82


class ActionId(GoProIntEnum):
    """Action ID's that identify a protobuf command."""

    SCAN_WIFI_NETWORKS = 0x02
    GET_AP_ENTRIES = 0x03
    REQUEST_WIFI_CONNECT = 0x04
    REQUEST_WIFI_CONNECT_NEW = 0x05
    NOTIF_START_SCAN = 0x0B
    NOTIF_PROVIS_STATE = 0x0C
    REQUEST_PRESET_UPDATE_CUSTOM = 0x64
    SET_CAMERA_CONTROL = 0x69
    SET_TURBO_MODE = 0x6B
    GET_PRESET_STATUS = 0x72
    GET_LIVESTREAM_STATUS = 0x74
    RELEASE_NETWORK = 0x78
    SET_LIVESTREAM_MODE = 0x79
    SCAN_WIFI_NETWORKS_RSP = 0x82
    GET_AP_ENTRIES_RSP = 0x83
    REQUEST_WIFI_CONNECT_RSP = 0x84
    REQUEST_WIFI_CONNECT_NEW_RSP = 0x85
    REQUEST_COHN_SETTING = 0x65
    REQUEST_CLEAR_COHN_CERT = 0x66
    REQUEST_CREATE_COHN_CERT = 0x67
    REQUEST_GET_LAST_MEDIA = 0x6D
    REQUEST_GET_COHN_CERT = 0x6E
    REQUEST_GET_COHN_STATUS = 0x6F
    RESPONSE_PRESET_UPDATE_CUSTOM = 0xE4
    RESPONSE_COHN_SETTING = 0xE5
    RESPONSE_CLEAR_COHN_CERT = 0xE6
    RESPONSE_CREATE_COHN_CERT = 0xE7
    SET_CAMERA_CONTROL_RSP = 0xE9
    SET_TURBO_MODE_RSP = 0xEB
    RESPONSE_GET_LAST_MEDIA = 0xED
    RESPONSE_GET_COHN_CERT = 0xEE
    RESPONSE_GET_COHN_STATUS = 0xEF
    GET_PRESET_STATUS_RSP = 0xF2
    PRESET_MODIFIED_NOTIFICATION = 0xF3
    LIVESTREAM_STATUS_RSP = 0xF4
    LIVESTREAM_STATUS_NOTIF = 0xF5
    RELEASE_NETWORK_RSP = 0xF8
    SET_LIVESTREAM_MODE_RSP = 0xF9
    INTERNAL_FF = 0xFF


class FeatureId(GoProIntEnum):
    """ID's that group protobuf commands"""

    NETWORK_MANAGEMENT = 0x02
    COMMAND = 0xF1
    SETTING = 0xF3
    QUERY = 0xF5


class QueryCmdId(GoProIntEnum):
    """Command ID that is written to GoProUUID.CQ_QUERY."""

    GET_SETTING_VAL = 0x12
    GET_STATUS_VAL = 0x13
    GET_SETTING_NAME = 0x22
    GET_CAPABILITIES_VAL = 0x32
    GET_CAPABILITIES_NAME = 0x42
    REG_SETTING_VAL_UPDATE = 0x52
    REG_STATUS_VAL_UPDATE = 0x53
    REG_CAPABILITIES_UPDATE = 0x62
    UNREG_SETTING_VAL_UPDATE = 0x72
    UNREG_STATUS_VAL_UPDATE = 0x73
    UNREG_CAPABILITIES_UPDATE = 0x82
    SETTING_VAL_PUSH = 0x92
    STATUS_VAL_PUSH = 0x93
    SETTING_CAPABILITY_PUSH = 0xA2
    PROTOBUF_QUERY = 0xF5


class WebcamStatus(GoProIntEnum):
    """Webcam Statuses / states"""

    OFF = 0
    IDLE = 1
    HIGH_POWER_PREVIEW = 2
    LOW_POWER_PREVIEW = 3


class WebcamError(GoProIntEnum):
    """Errors common among Webcam commands"""

    SUCCESS = 0
    SET_PRESET = 1
    SET_WINDOW_SIZE = 2
    EXEC_STREAM = 3
    SHUTTER = 4
    COM_TIMEOUT = 5
    INVALID_PARAM = 6
    UNAVAILABLE = 7
    EXIT = 8


class Toggle(GoProIntEnum):
    """A booleanesque enum"""

    ENABLE = 1
    DISABLE = 0


class LED_SPECIAL(GoProIntEnum):
    """Special (not functional) LED value used for keep alive signal"""

    BLE_KEEP_ALIVE = 66


class CameraControl(GoProIntEnum):
    """Camera Control Request Values"""

    IDLE = 0
    CAMERA = 1
    EXTERNAL = 2


class WebcamResolution(GoProIntEnum):
    """Possible Webcam Resolutions"""

    NOT_APPLICABLE = 0
    RES_480 = 4
    RES_720 = 7
    RES_1080 = 12


class WebcamFOV(GoProIntEnum):
    """Possible Webcam FOVs"""

    WIDE = 0
    NARROW = 2
    SUPERVIEW = 3
    LINEAR = 4


class WebcamProtocol(GoProEnum):
    """Possible Webcam Protocols"""

    TS = "TS"
    RTSP = "RTSP"



# __init__.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Feb  3 00:30:25 UTC 2022

"""Top level API module definition"""

from .api import WiredApi, WirelessApi
from .ble_commands import BleCommands
from .ble_settings import BleSettings
from .ble_statuses import BleStatuses
from .builders import (
    BleAsyncResponse,
    BleProtoCommand,
    BleReadCommand,
    BleSettingFacade,
    BleStatusFacade,
    BleWriteCommand,
    HttpSetting,
    RegisterUnregisterAll,
)
from .http_commands import HttpCommands
from .http_settings import HttpSettings

# We need to ensure the API instantiated so that all parsers are set up.
WirelessApi(None)  # type: ignore



# communication_client.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Tue Sep  7 21:35:53 UTC 2021

"""GoPro specific BLE client"""

from __future__ import annotations

import enum
import inspect
import logging
import re
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Any, Generic, Iterator, Pattern, Protocol, TypeVar
from urllib.parse import urlencode

from construct import Bit, BitsInteger, BitStruct, Const, Construct, Padding

from open_gopro.domain.parser_interface import (
    BytesParser,
    BytesTransformer,
    GlobalParsers,
    JsonParser,
    JsonTransformer,
    Parser,
)
from open_gopro.domain.types import IdType, JsonDict, UpdateCb, UpdateType
from open_gopro.models import GoProBlePacketHeader, GoProResp
from open_gopro.models.constants import GoProUUID
from open_gopro.network.ble import (
    BleClient,
    BLEController,
    BleDevice,
    BleHandle,
    BleUUID,
    DisconnectHandlerType,
    NotiHandlerType,
)
from open_gopro.network.wifi import WifiClient, WifiController

logger = logging.getLogger(__name__)


class MessageRules:
    """Message Rules Manager

    Attributes:
        always_false (Analyzer): helper analyzer for a property that is always false
        always_true (Analyzer): helper analyzer for a property that is always true

    Args:
        fastpass_analyzer (Analyzer): Analyzer to decide if the message is fastpass. Defaults to always_false.
        wait_for_encoding_analyzer (Analyzer): Analyzer to decide if the message should wait for encoding.
            Defaults to always_false.
    """

    class Analyzer(Protocol):
        """Protocol definition of message rules analyzer"""

        def __call__(self, **kwargs: Any) -> bool:
            """Analyze the current inputs to see if the rule should be applied

            Args:
                **kwargs (Any): input arguments

            Returns:
                bool: Should the rule be applied?
            """

    always_false: Analyzer = lambda **kwargs: False
    always_true: Analyzer = lambda **kwargs: True

    def __init__(
        self, fastpass_analyzer: Analyzer = always_false, wait_for_encoding_analyzer: Analyzer = always_false
    ) -> None:
        self._analyze_fastpass = fastpass_analyzer
        self._analyze_wait_for_encoding = wait_for_encoding_analyzer

    def is_fastpass(self, **kwargs: Any) -> bool:
        """Is this command fastpass?

        Args:
            **kwargs (Any) : Arguments passed into the message

        Returns:
            bool: result of rule check
        """
        return self._analyze_fastpass(**kwargs)

    def should_wait_for_encoding_start(self, **kwargs: Any) -> bool:
        """Should this message wait for encoding to start?

        Args:
            **kwargs (Any) : Arguments passed into the message

        Returns:
            bool: result of rule check
        """
        return self._analyze_wait_for_encoding(**kwargs)


##############################################################################################################
####### Communicators / Clients
##############################################################################################################


class BaseGoProCommunicator(ABC):
    """Common Communicator interface"""

    class _CompositeRegisterType(enum.Enum):
        """A composite register type to signify all elements of a given set should be (un)registered for"""

        ALL_SETTINGS = enum.auto()
        ALL_STATUSES = enum.auto()

    @abstractmethod
    def _register_update(self, callback: UpdateCb, update: _CompositeRegisterType | UpdateType) -> None:
        """Common register method for both public UpdateType and "protected" internal register type

        Args:
            callback (UpdateCb): callback to register
            update (_CompositeRegisterType | UpdateType): update type to register for
        """

    @abstractmethod
    def _unregister_update(self, callback: UpdateCb, update: _CompositeRegisterType | UpdateType | None = None) -> None:
        """Common unregister method for both public UpdateType and "protected" internal register type

        Args:
            callback (UpdateCb): callback to unregister
            update (_CompositeRegisterType | UpdateType | None): Update type to unregister for. Defaults to
                    None which will unregister the callback for all update types.
        """

    @property
    @abstractmethod
    def identifier(self) -> str:
        """_summary_

        Returns:
            str: _description_
        """

    @abstractmethod
    def register_update(self, callback: UpdateCb, update: UpdateType) -> None:
        """Register for callbacks when an update occurs

        Args:
            callback (UpdateCb): callback to be notified in
            update (UpdateType): update to register for
        """

    @abstractmethod
    def unregister_update(self, callback: UpdateCb, update: UpdateType | None = None) -> None:
        """Unregister for asynchronous update(s)

        Args:
            callback (UpdateCb): callback to stop receiving update(s) on
            update (UpdateType | None): updates to unsubscribe for. Defaults to None (all
                updates that use this callback will be unsubscribed).
        """


class GoProHttp(BaseGoProCommunicator):
    """Interface definition for all HTTP communicators"""

    @abstractmethod
    async def _get_json(
        self, message: HttpMessage, *, timeout: int = 0, rules: MessageRules | None = MessageRules(), **kwargs: Any
    ) -> GoProResp:
        """Perform a GET operation that returns JSON

        Args:
            message (HttpMessage): operation description
            timeout (int): time (in seconds) to wait to receive response before returning error. Defaults to 0.
            rules (MessageRules | None): message rules that this operation will obey. Defaults to MessageRules().
            **kwargs (Any) : any run-time arguments to apply to the operation

        Returns:
            GoProResp: response parsed from received JSON
        """

    @abstractmethod
    async def _get_stream(
        self, message: HttpMessage, *, timeout: int = 0, rules: MessageRules | None = MessageRules(), **kwargs: Any
    ) -> GoProResp:
        """Perform a GET operation that returns a binary stream

        Args:
            message (HttpMessage): operation description
            timeout (int): time (in seconds) to wait to receive response before returning error. Defaults to 0.
            rules (MessageRules | None): message rules that this operation will obey. Defaults to MessageRules().
            **kwargs (Any) : any run-time arguments to apply to the operation

        Returns:
            GoProResp: response wrapper around downloaded binary
        """

    @abstractmethod
    async def _put_json(
        self, message: HttpMessage, *, timeout: int = 0, rules: MessageRules | None = MessageRules(), **kwargs: Any
    ) -> GoProResp:
        """Perform a PUT operation that returns JSON

        Args:
            message (HttpMessage): operation description
            timeout (int): time (in seconds) to wait to receive response before returning error. Defaults to 0.
            rules (MessageRules | None): message rules that this operation will obey. Defaults to MessageRules().
            **kwargs (Any) : any run-time arguments to apply to the operation

        Returns:
            GoProResp: response parsed from received JSON
        """


class GoProWifi(GoProHttp):
    """GoPro specific WiFi Client

    Args:
        controller (WifiController): instance of Wifi Controller to use for this client
    """

    def __init__(self, controller: WifiController):
        self._wifi: WifiClient = WifiClient(controller)

    @property
    def password(self) -> str | None:
        """Get the GoPro AP's password

        Returns:
            str | None: password or None if it is not known
        """
        return self._wifi.password

    @property
    def ssid(self) -> str | None:
        """Get the GoPro AP's WiFi SSID

        Returns:
            str | None: SSID or None if it is not known
        """
        return self._wifi.ssid


class GoProBle(BaseGoProCommunicator, Generic[BleHandle, BleDevice]):
    """GoPro specific BLE Client

    Args:
        controller (BLEController): controller implementation to use for this client
        disconnected_cb (DisconnectHandlerType): disconnected callback
        notification_cb (NotiHandlerType): notification callback
        target (str | BleDevice): device to connect to or trailing digits of serial number
    """

    def __init__(
        self,
        controller: BLEController,
        disconnected_cb: DisconnectHandlerType,
        notification_cb: NotiHandlerType,
        target: str | BleDevice,
    ) -> None:
        self._ble: BleClient = BleClient(
            controller,
            disconnected_cb,
            notification_cb,
            (re.compile(r"GoPro [A-Z0-9]{4}") if target is None else f".*{target}", [GoProUUID.S_CONTROL_QUERY]),
            uuids=GoProUUID,
        )

    @abstractmethod
    async def _send_ble_message(
        self, message: BleMessage, rules: MessageRules = MessageRules(), **kwargs: Any
    ) -> GoProResp:
        """Perform a GATT write with response and accumulate received notifications into a response.

        Args:
            message (BleMessage): BLE operation description
            rules (MessageRules): message rules that this operation will obey. Defaults to MessageRules().
            **kwargs (Any) : any run-time arguments to apply to the operation

        Returns:
            GoProResp: response parsed from accumulated BLE notifications
        """

    @abstractmethod
    async def _read_ble_characteristic(
        self, message: BleMessage, rules: MessageRules = MessageRules(), **kwargs: Any
    ) -> GoProResp:
        """Perform a direct GATT read of a characteristic

        Args:
            message (BleMessage): BLE operation description
            rules (MessageRules): message rules that this operation will obey. Defaults to MessageRules().
            **kwargs (Any) : any run-time arguments to apply to the operation

        Returns:
            GoProResp: response parsed from bytes read from characteristic
        """

    # TODO this should be somewhere else
    @classmethod
    def _fragment(cls, data: bytes) -> Iterator[bytes]:
        """Fragment data in to MAX_BLE_PKT_LEN length packets

        Args:
            data (bytes): data to fragment

        Raises:
            ValueError: data is too long

        Yields:
            bytes: packet as bytes
        """
        MAX_BLE_PKT_LEN = 20

        extended_13_header = BitStruct(
            "continuation" / Const(0, Bit),
            "header" / Const(GoProBlePacketHeader.EXT_13.value, BitsInteger(2)),
            "length" / BitsInteger(13),
        )

        extended_16_header = BitStruct(
            "continuation" / Const(0, Bit),
            "header" / Const(GoProBlePacketHeader.EXT_16.value, BitsInteger(2)),
            "padding" / Padding(5),
            "length" / BitsInteger(16),
        )

        continuation_header = BitStruct(
            "continuation" / Const(1, Bit),
            "padding" / Padding(7),
        )

        header: Construct
        if (data_len := len(data)) < (2**13 - 1):
            header = extended_13_header
        elif data_len < (2**16 - 1):
            header = extended_16_header
        else:
            raise ValueError(f"Data length {data_len} is too long")

        assert header
        while data:
            if header == continuation_header:
                packet = bytearray(header.build({}))
            else:
                packet = bytearray(header.build({"length": data_len}))
                header = continuation_header

            bytes_remaining = MAX_BLE_PKT_LEN - len(packet)
            current, data = (data[:bytes_remaining], data[bytes_remaining:])
            packet.extend(current)
            yield bytes(packet)


class GoProWiredInterface(BaseGoProCommunicator):
    """The top-level interface for a Wired Open GoPro controller"""


class GoProWirelessInterface(GoProBle, GoProWifi, Generic[BleDevice, BleHandle]):
    """The top-level interface for a Wireless Open GoPro controller

    This always supports BLE and can optionally support Wifi

    Args:
        ble_controller (BLEController): BLE controller instance
        wifi_controller (WifiController): Wifi controller instance
        disconnected_cb (DisconnectHandlerType): callback for BLE disconnects
        notification_cb (NotiHandlerType): callback for BLE received notifications
        target (Pattern | BleDevice): BLE device to search for
    """

    def __init__(
        self,
        ble_controller: BLEController,
        wifi_controller: WifiController,
        disconnected_cb: DisconnectHandlerType,
        notification_cb: NotiHandlerType,
        target: Pattern | BleDevice,
    ) -> None:
        # Initialize GoPro Communication Client
        GoProBle.__init__(self, ble_controller, disconnected_cb, notification_cb, target)
        GoProWifi.__init__(self, wifi_controller)


ParserType = TypeVar("ParserType", BytesParser, JsonParser)
FilterType = TypeVar("FilterType", BytesTransformer, JsonTransformer)


##############################################################################################################
####### Messages (commands, etc.)
##############################################################################################################


class Message(ABC):
    """Base class for all messages that will be contained in a Messages class"""

    def __init__(
        self,
        identifier: IdType,
        parser: Parser | None = None,
    ) -> None:
        self._identifier: IdType = identifier
        self._parser = parser

    @abstractmethod
    def _as_dict(self, **kwargs: Any) -> JsonDict:
        """Return the attributes of the message as a dict

        Args:
            **kwargs (Any): additional entries for the dict

        Returns:
            JsonDict: message as dict
        """


class BleMessage(Message):
    """The base class for all BLE messages to store common info

    Args:
        uuid (BleUUID): BLE client to read / write
        identifier (IdType): BleUUID to read / write to
        parser (Parser | None): parser to interpret message
    """

    def __init__(
        self,
        uuid: BleUUID,
        identifier: IdType,
        parser: Parser | None,
    ) -> None:
        Message.__init__(self, identifier, parser)
        self._uuid = uuid
        self._base_dict = {"protocol": GoProResp.Protocol.BLE, "uuid": self._uuid}

        if parser:
            GlobalParsers.add(identifier, parser)

    @abstractmethod
    def _build_data(self, **kwargs: Any) -> bytearray:
        """Build the raw write request from operation description and run-time arguments

        Args:
            **kwargs (Any) : run-time arguments

        Returns:
            bytearray: raw bytes request
        """


class HttpMessage(Message):
    """The base class for all HTTP messages. Stores common information.

    Args:
        endpoint (str): base endpoint
        identifier (IdType | None): explicit message identifier. If None, will be generated from endpoint.
        components (list[str] | None): Additional path components (i.e. endpoint/{COMPONENT}). Defaults to None.
        arguments (list[str] | None): Any arguments to be appended after endpoint (i.e. endpoint?{ARGUMENT}). Defaults to None.
        body_args (list[str] | None): Arguments to be added to the body JSON. Defaults to None.
        headers (dict[str, Any] | None): A dict of values to be set in the HTTP operation headers. Defaults to None.
        certificate (Path | None): Path to SSL certificate bundle. Defaults to None.
        parser (Parser | None): Parser to interpret HTTP responses. Defaults to None.
    """

    def __init__(
        self,
        endpoint: str,
        identifier: IdType | None,
        components: list[str] | None = None,
        arguments: list[str] | None = None,
        body_args: list[str] | None = None,
        headers: dict[str, Any] | None = None,
        certificate: Path | None = None,
        parser: Parser | None = None,
    ) -> None:
        if not identifier:
            # Build human-readable name from endpoint
            identifier = endpoint.lower().removeprefix("gopro/").replace("/", " ").replace("_", " ").title()
            try:
                identifier = identifier.split("?")[0].strip("{}")
            except IndexError:
                pass

        self._headers = headers or {}
        self._endpoint = endpoint
        self._components = components or []
        self._arguments = arguments or []
        self._body_args = body_args or []
        self._certificate = certificate
        Message.__init__(self, identifier, parser)
        self._base_dict: JsonDict = {
            "id": self._identifier,
            "protocol": GoProResp.Protocol.HTTP,
            "endpoint": self._endpoint,
        }

    def __str__(self) -> str:
        return str(self._identifier).title()

    def _as_dict(self, **kwargs: Any) -> JsonDict:
        """Return the attributes of the message as a dict

        Args:
            **kwargs (Any): additional entries for the dict

        Returns:
            JsonDict: message as dict
        """
        # If any kwargs keys were to conflict with base dict, append underscore
        return self._base_dict | {f"{'_' if k in ['id', 'protocol'] else ''}{k}": v for k, v in kwargs.items()}

    def build_body(self, **kwargs: Any) -> dict[str, Any]:
        """Build JSON body from run-time body arguments

        Args:
            **kwargs (Any): run-time arguments to check to see if each should be added to the body

        Returns:
            dict[str, Any]: built JSON body
        """
        body: dict[str, Any] = {}
        for name, value in kwargs.items():
            if name in self._body_args:
                body[name] = value
        return body

    def build_url(self, **kwargs: Any) -> str:
        """Build the URL string from the passed in components and arguments

        Args:
            **kwargs (Any): additional entries for the dict

        Returns:
            str: built URL
        """
        url = self._endpoint
        for component in self._components:
            url += "/" + kwargs.pop(component)
        # Append parameters
        if self._arguments and (
            arg_part := urlencode(
                {
                    k: kwargs[k].value if isinstance(kwargs[k], enum.Enum) else kwargs[k]
                    for k in self._arguments
                    if kwargs[k] is not None
                },
                safe="/",
            )
        ):
            url += "?" + arg_part
        return url


MessageType = TypeVar("MessageType", bound=Message)

CommunicatorType = TypeVar("CommunicatorType", bound=BaseGoProCommunicator)


class Messages(ABC, dict, Generic[MessageType, CommunicatorType]):
    """Base class for setting and status containers

    Allows message groups to be iterable and supports dict-like access.

    Instance attributes that are an instance (or subclass) of Message are automatically accumulated during
    instantiation

    Args:
        communicator (CommunicatorType): communicator that will send messages
    """

    def __init__(self, communicator: CommunicatorType) -> None:
        self._communicator = communicator
        # Append any automatically discovered instance attributes (i.e. for settings and statuses)
        message_map: dict[IdType, MessageType] = {}
        for message in self.__dict__.values():
            if hasattr(message, "_identifier"):
                message_map[message._identifier] = message
        # Append any automatically discovered methods (i.e. for commands)
        for name, method in inspect.getmembers(self, predicate=inspect.ismethod):
            if not name.startswith("_"):
                message_map[name.replace("_", " ").title()] = method  # type: ignore
        dict.__init__(self, message_map)


class BleMessages(Messages[MessageType, GoProBle]):
    """A container of BLE Messages.

    Identical to Messages and it just used for typing
    """


class HttpMessages(Messages[MessageType, GoProHttp]):
    """A container of HTTP Messages.

    Identical to Messages and it just used for typing
    """



# cohn_feature.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon May 12 23:03:50 UTC 2025

"""Camera on the home network (COHN) feature abstraction."""

from __future__ import annotations

import asyncio
import logging

from returns.pipeline import is_successful
from returns.result import Result
from tinydb import TinyDB

from open_gopro.api import WirelessApi
from open_gopro.database.cohn_db import CohnDb
from open_gopro.domain.exceptions import GoProNotOpened
from open_gopro.domain.gopro_observable import GoProObservable
from open_gopro.features.base_feature import BaseFeature
from open_gopro.gopro_base import GoProBase
from open_gopro.models import proto
from open_gopro.models.constants import ActionId
from open_gopro.models.general import CohnInfo
from open_gopro.models.proto import EnumCOHNNetworkState, EnumCOHNStatus

logger = logging.getLogger(__name__)


class CohnFeature(BaseFeature):
    """Camera on the home network (COHN) feature abstraction

    Args:
        cohn_db (TinyDB): TinyDB database to use for COHN credentials
        gopro (GoProBase[WirelessApi]): camera to operate on
        loop (asyncio.AbstractEventLoop): asyncio loop to use for this feature
        cohn_credentials (CohnInfo | None): COHN credentials to use for this camera. Defaults to None in
            which case they will be retrieved from DB / connected camera.
    """

    def __init__(
        self,
        cohn_db: TinyDB,
        gopro: GoProBase[WirelessApi],
        loop: asyncio.AbstractEventLoop,
        cohn_credentials: CohnInfo | None = None,
    ) -> None:
        super().__init__(gopro, loop)
        self._db = CohnDb(cohn_db)
        if cohn_credentials:
            self.credentials = cohn_credentials
        self._ready_event = asyncio.Event()
        # TODO close this
        self._status_observable = GoProObservable(
            gopro=self._gopro,
            update=ActionId.RESPONSE_GET_COHN_STATUS,
            register_command=self._gopro.ble_command.cohn_get_status(register=True),
        ).on_start(lambda _: self._ready_event.set())
        self._status_task: asyncio.Task = asyncio.create_task(self._track_status())

    @property
    def is_ready(self) -> bool:  # noqa: D102
        return self._ready_event.is_set()

    async def wait_for_ready(self, timeout: float = 60) -> None:  # noqa: D102
        logger.debug("Waiting for COHN to be ready")
        await asyncio.wait_for(self._ready_event.wait(), timeout)
        logger.debug("COHN is ready")

    def close(self) -> None:  # noqa: D102
        self._status_task.cancel()

    async def _track_status(self) -> None:
        """Task to continuously monitor COHN status"""
        while True:
            if self._gopro.is_ble_connected:
                async with self._status_observable as observable:
                    async for status in observable.observe(debug_id="Cohn Feature"):
                        logger.debug(f"Feature Received COHN status: {status}")
            else:
                await asyncio.sleep(1)

    @property
    def credentials(self) -> CohnInfo | None:
        """The camera's COHN credentials

        Returns:
            CohnInfo | None: Credentials if they exist or None if they have not yet been discovered
        """
        return self._db.search_credentials(self._gopro.identifier)

    @credentials.setter
    def credentials(self, new_credentials: CohnInfo) -> None:
        self._db.insert_or_update_credentials(self._gopro.identifier, new_credentials)

    @property
    def status(self) -> proto.NotifyCOHNStatus:
        """The current COHN status

        Raises:
            GoProNotOpened: There is no status yet because the camera is not yet ready.

        Returns:
            proto.NotifyCOHNStatus: The current COHN status
        """
        if not self.is_ready or not self._status_observable.current:
            raise GoProNotOpened("COHN feature is not yet ready")
        return self._status_observable.current

    @property
    async def is_configured(self) -> bool:
        """Is COHN ready for communication?

        This will check if there are credentials and that validate that a basic HTTP operations works.

        Returns:
            bool: True if configured, False otherwise
        """
        return bool(self.credentials and self.credentials.is_complete)

    @property
    def is_connected(self) -> bool:
        """Is COHN connected to an Access Point?

        Returns:
            bool: True if connected, False otherwise
        """
        return self.status.state == EnumCOHNNetworkState.COHN_STATE_NetworkConnected

    @property
    def _is_provisioned(self) -> bool:
        """Is COHN provisioned?

        Returns:
            bool: True if provisioned, False otherwise
        """
        return self.status.status == EnumCOHNStatus.COHN_PROVISIONED

    async def _provision_cohn(self, timeout: int = 60) -> Result[CohnInfo, TimeoutError]:
        """Provision the camera, clearing any current certificate and forcing reprovision

        Args:
            timeout (int): How long in seconds to wait before considering the provision a failure.
                Defaults to 60.

        Raises:
            GoProNotOpened: BLE is not available

        Returns:
            Result[CohnInfo, TimeoutError]: COHN Credentials if provisioning succeeds, otherwise an error
        """
        if not self._gopro.is_ble_connected:
            raise GoProNotOpened("COHN needs to be provisioned but BLE is not connected")

        logger.info("Provisioning COHN")
        try:
            async with asyncio.timeout(timeout):
                # Start fresh by clearing cert and wait until we receive unprovisioned status
                await self._gopro.ble_command.cohn_clear_certificate()
                await self._status_observable.observe().first(
                    lambda status: status.status == EnumCOHNStatus.COHN_UNPROVISIONED
                )
                logger.info("COHN has been unprovisioned")

                # Reprovision and wait until we receive provisioned status
                assert (await self._gopro.ble_command.cohn_create_certificate(override=True)).ok
                status = await self._status_observable.observe().first(
                    lambda status: status.status == EnumCOHNStatus.COHN_PROVISIONED
                )
                logger.info("COHN has been successfully provisioned!!")

                cert = (await self._gopro.ble_command.cohn_get_certificate()).data.cert
                credentials = CohnInfo(
                    ip_address=status.ipaddress,
                    username=status.username,
                    password=status.password,
                    certificate=cert,
                )
                self.credentials = credentials
                return Result.from_value(credentials)
        except TimeoutError as exc:
            return Result.from_failure(exc)

    async def configure(
        self,
        force_reprovision: bool = False,
        timeout: int = 60,
    ) -> Result[CohnInfo, RuntimeError | TimeoutError]:
        """Prepare Camera on the Home Network

        Provision if not provisioned
        Then wait for COHN to be connected and ready

        Args:
            force_reprovision (bool): Set to True to force reprovisioning. Defaults to False.
            timeout (int): time in seconds to wait for COHN to be ready. Defaults to 60.

        Returns:
            Result[CohnInfo, RuntimeError | TimeoutError]: _description_
        """
        # If we don't have credentials or peer isn't provisioned, we need to (re)provision
        if force_reprovision or not await self.is_configured:
            if not is_successful(result := await self._provision_cohn()):
                return result
        # We should at least have incomplete credentials after provisioning
        assert self.credentials
        try:
            if not self.is_connected:
                logger.info("Waiting for COHN to be connected")

                async with asyncio.timeout(timeout):
                    await self._status_observable.observe().first(
                        lambda s: s.state == EnumCOHNNetworkState.COHN_STATE_NetworkConnected
                    )

            logger.info("COHN is connected")
            # On some cameras, the IP address only comes with this status. Let's just always take all of the available
            # information from the current status which was potentially just retrieved from connecting.
            self.credentials = CohnInfo(
                username=self.status.username,
                password=self.status.password,
                ip_address=self.status.ipaddress,
                certificate=self.credentials.certificate,
            )

        except TimeoutError as exc:
            return Result.from_failure(exc)
        # If the credentials are still not complete at this point, something bad has happened
        if not self.credentials.is_complete:
            logger.error("Failed to get COHN credentials")
            return Result.from_failure(RuntimeError("Failed to get COHN credentials"))
        logger.info("COHN is provisioned, connected, and ready for communication")
        return Result.from_value(self.credentials)



# __init__.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon Jun 26 18:26:05 UTC 2023

"""Data models for use throughout this package"""

from .general import (
    CameraInfo,
    CohnInfo,
    HttpInvalidSettingResponse,
    ScheduledCapture,
    TzDstDateTime,
    WebcamResponse,
)
from .media_list import (
    GroupedMediaItem,
    MediaItem,
    MediaList,
    MediaMetadata,
    MediaPath,
    PhotoMetadata,
    VideoMetadata,
)
from .network_scan_responses import AdvData, DnsScanResponse, GoProAdvData
from .response import GoProBlePacketHeader, GoProResp



# util.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Wed, Sep  1, 2021  5:05:50 PM

"""Miscellaneous utilities for the GoPro package."""

from __future__ import annotations

import argparse
import asyncio
import enum
import logging
import subprocess
import sys
from dataclasses import is_dataclass
from datetime import datetime
from pathlib import Path
from typing import TYPE_CHECKING, Any, Callable, Generic, TypeVar

import pytz
from construct import Container
from pydantic import BaseModel
from typing_extensions import TypeIs
from tzlocal import get_localzone

if TYPE_CHECKING:
    from _typeshed import DataclassInstance


util_logger = logging.getLogger(__name__)


class Singleton:
    """To be subclassed to create a singleton class."""

    _instances: dict[type[Singleton], Singleton] = {}

    def __new__(cls, *_: Any) -> Any:  # noqa https://github.com/PyCQA/pydocstyle/issues/515
        if cls not in cls._instances:
            cls._instances[cls] = object.__new__(cls)
        return cls._instances[cls]


def map_keys(obj: Any, key: str, func: Callable) -> None:
    """Map all matching keys (deeply searched) using the input function

    Args:
        obj (Any): object to modify in place
        key (str): key to search for to modify
        func (Callable): mapping function
    """
    if isinstance(obj, dict):
        for k in obj.keys():
            if k == key:
                obj[k] = func(obj[k])
            else:
                map_keys(obj[k], key, func)
    elif isinstance(obj, list):
        for i in obj:
            map_keys(i, key, func)
    else:
        # neither a dict nor a list, do nothing
        pass


def scrub(obj: Any, bad_keys: list | None = None, bad_values: list | None = None) -> None:
    """Recursively scrub a collection (dict / list) of bad keys and / or bad values

    Args:
        obj (Any): collection to scrub
        bad_keys (list | None): Keys to remove. Defaults to None.
        bad_values (list | None): Values to remove. Defaults to None.

    Raises:
        ValueError: Missing bad keys / values
    """
    bad_keys = bad_keys or []
    bad_values = bad_values or []
    if not (bad_values or bad_keys):
        raise ValueError("Must pass either / or bad_keys or bad_values")

    def recurse(obj: Any) -> None:
        if isinstance(obj, dict):
            for key, value in {**obj}.items():
                if key in bad_keys or value in bad_values:
                    del obj[key]
                else:
                    recurse(obj[key])
        elif isinstance(obj, list):
            for i, value in enumerate(list(obj)):
                if value in bad_values:
                    del obj[i]
                else:
                    recurse(obj[i])
        else:
            # neither a dict nor a list, do nothing
            pass

    recurse(obj)


def pretty_print(obj: Any, stringify_all: bool = True, should_quote: bool = True) -> str:
    """Recursively iterate through object and turn elements into strings

    Args:
        obj (Any): object to recurse through
        stringify_all (bool): At the end of each recursion, should the element be turned into a string?
            For example, should an int be turned into a str? Defaults to True.
        should_quote (bool): Should each element be surrounded in quotes?. Defaults to True.

    Returns:
        str: pretty-printed string
    """
    output = ""
    nest_level = 0

    def sanitize(e: Any) -> str:
        """Get the value part and replace any underscored with spaces

        Args:
            e (Any): argument to sanitize

        Returns:
            str: sanitized string
        """
        value_part = str(e).lower().split(".")[1]
        value_part = value_part.replace("_", " ").title()
        return value_part

    def stringify(elem: Any) -> Any:
        """Get the string value of an element if it is not a number (int, float, etc.)

        Args:
            elem (Any): element to potentially stringify

        Returns:
            Any: string representation or original object
        """

        def quote(elem: Any) -> Any:
            return f'"{elem}"' if should_quote else elem

        ret: str
        if isinstance(elem, (bytes, bytearray)):
            ret = quote(elem.hex(":"))
        if isinstance(elem, enum.Enum) and isinstance(elem, int):
            ret = quote(str(elem) if not stringify_all else sanitize(elem))
        if isinstance(elem, (bool, int, float)):
            ret = quote(elem) if stringify_all else elem  # type: ignore
        ret = str(elem)
        return quote(ret)

    def recurse(elem: Any) -> None:
        """Recursion function

        Args:
            elem (Any): current element to work on
        """
        nonlocal output
        nonlocal nest_level
        indent_size = 4
        # Convert to dict if possible
        if isinstance(elem, BaseModel):
            elem = dict(elem)
            scrub(elem, bad_values=[None])
        if isinstance(elem, dict):
            # nested dictionary
            nest_level += 1
            output += "{"
            for k, v in elem.items():
                output += f"\n{' ' * (indent_size * nest_level)}"
                # Add key
                recurse(k)
                output += " : "
                # Add value
                if isinstance(v, (dict, list, BaseModel)):
                    recurse(v)
                else:
                    output += stringify(v)
                output += ","

            nest_level -= 1
            output += f"\n{' '* (indent_size * nest_level)}}}"

        elif isinstance(elem, list):
            # nested list
            nest_level += 1
            output += f"[\n{' '* (indent_size * nest_level)}"
            if len(elem):
                for item in elem[:-1]:
                    recurse(item)
                    output += ", "
                recurse(elem[-1])
            nest_level -= 1
            output += f"\n{' '* (indent_size * nest_level)}]"

        else:
            output += stringify(elem)

    recurse(obj)
    return output


def cmd(command: str) -> str:
    """Send a command to the shell and return the result.

    Args:
        command (str): command to send

    Returns:
        str: response returned from shell
    """
    # We don't want password showing in the log
    if "sudo" in command:
        logged_command = command[: command.find('"') + 1] + "********" + command[command.find(" | sudo") - 1 :]
    else:
        logged_command = command
    util_logger.debug(f"Send cmd --> {logged_command}")
    response = (
        subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)  # type: ignore
        .stdout.read()
        .decode(errors="ignore")
    )
    util_logger.debug(f"Receive response --> {response}")

    return response


T = TypeVar("T")


class SnapshotQueue(asyncio.Queue, Generic[T]):
    """A subclass of the default queue module to safely take a snapshot of the queue

    This is so we can access the elements (in a thread safe manner) without dequeuing them.
    """

    def __init__(self, maxsize: int = 0) -> None:
        self._lock = asyncio.Lock()
        super().__init__(maxsize)

    async def get(self) -> T:
        """Wrapper for passing generic type through to subclass

        Returns:
            T: type of this Snapshot queue
        """
        return await super().get()

    async def peek_front(self) -> T | None:
        """Get the first element without dequeueing it

        Returns:
            T | None: First element of None if the queue is empty
        """
        async with self._lock:
            return None if self.empty() else self._queue[0]  # type: ignore


def add_cli_args_and_parse(
    parser: argparse.ArgumentParser,
    bluetooth: bool = True,
    wifi: bool = True,
) -> argparse.Namespace:
    """Append common argparse arguments to an argument parser

    WARNING!! This will also parse the arguments (i.e. call parser.parse_args) so ensure to add any additional
    arguments to the parser before passing it to this function.

    Args:
        parser (argparse.ArgumentParser): input parser to modify
        bluetooth (bool): Add bluetooth args?. Defaults to True.
        wifi (bool): Add WiFi args?. Defaults to True.

    Returns:
        argparse.Namespace: modified argument parser
    """
    # Common args
    parser.add_argument(
        "--log",
        type=Path,
        help="Location to store detailed log. Defaults to gopro_demo.log",
        default="gopro_demo.log",
    )

    if bluetooth:
        parser.add_argument(
            "--identifier",
            type=str,
            help="Last 4 digits of GoPro serial number, which is the last 4 digits of the default camera SSID. \
                If not used, first discovered GoPro will be connected to",
            default=None,
        )

    if wifi:
        parser.add_argument(
            "--wifi_interface",
            type=str,
            help="System Wifi Interface. If not set, first discovered interface will be used.",
            default=None,
        )
        parser.add_argument(
            "--password",
            action="store_true",
            help="Set to read sudo password from stdin. If not set, you will be prompted for password if needed",
        )

    parser.epilog = "Note that a minimal log is written to stdout. An extremely detailed log is written to the path set by the --log argument."
    args = parser.parse_args()
    if wifi:
        args.password = sys.stdin.readline() if args.password else None

    return args


async def ainput(string: str, printer: Callable | None = None) -> str:
    """Async version of input

    Raises:
        ValueError: Can not access default sys.stdout.write

    Args:
        string (str): prompt string
        printer (Callable | None): Printer used to display prompt. Defaults to None in which case sys.stdout.write
            will attempt to be used.

    Returns:
        str: Input read from console
    """
    if not printer:
        try:
            printer = sys.stdout.write
        except AttributeError as e:
            raise ValueError("No printer was passed and default standard out writer does not exist.") from e
    await asyncio.get_event_loop().run_in_executor(None, lambda s=string: printer(s + " "))  # type: ignore
    return await asyncio.get_event_loop().run_in_executor(None, sys.stdin.readline)


def get_current_dst_aware_time() -> tuple[datetime, int, bool]:
    """Get the current time, utc offset in minutes, and daylight savings time

    Returns:
        tuple[datetime, int, bool]: [time, utc_offset in minutes, is_dst?]
    """
    tz = pytz.timezone(get_localzone().key)  # type: ignore
    now = tz.localize(datetime.now(), is_dst=None)
    try:
        is_dst = now.tzinfo._dst.seconds != 0  # type: ignore
        offset = (now.utcoffset().total_seconds() - now.tzinfo._dst.seconds) / 60  # type: ignore
    except AttributeError:
        is_dst = False
        offset = now.utcoffset().total_seconds() / 60  # type: ignore
    if is_dst:
        offset += 60
    return (now, int(offset), is_dst)


def deeply_update_dict(d: dict, u: dict) -> dict:
    """Recursively update a dict

    Args:
        d (dict): original dict
        u (dict): dict to apply updates from

    Returns:
        dict: updated original dict
    """
    for k, v in u.items():
        if isinstance(v, dict):
            d[k] = deeply_update_dict(d.get(k, {}), v)
        else:
            d[k] = v
    return d


def to_dict(container: Container) -> dict:
    """Convert a parsed construct container to a dict, removing any internal Construct fields

    This is needed because annoyingly all construct's contain an "_io" field.
    See https://github.com/construct/construct/issues/1055

    Args:
        container (Container): container to convert

    Returns:
        dict: converted dict with any construct internal properties removed
    """
    d = dict(container)
    d.pop("_io", None)
    return d


def is_dataclass_instance(obj: Any) -> TypeIs[DataclassInstance | type[DataclassInstance]] | bool:
    """Check if a given object is a dataclass instance

    Args:
        obj (Any): object to analyze

    Returns:
        TypeIs[DataclassInstance | type[DataclassInstance]] | bool: TypeIs from analysis
    """
    return is_dataclass(obj) and not isinstance(obj, type)



# __init__.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon May 12 23:03:50 UTC 2025

"""Logger and other utility functionality."""

from .util import (
    Singleton,
    SnapshotQueue,
    add_cli_args_and_parse,
    ainput,
    cmd,
    deeply_update_dict,
    get_current_dst_aware_time,
    is_dataclass_instance,
    map_keys,
    pretty_print,
    scrub,
    to_dict,
)



# statuses.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Feb 20 23:24:52 UTC 2025

"""Status-related constants"""

########################################################################################################################
#
# Warning!! This file is auto-generated. Do not modify it manually.
#
########################################################################################################################

from open_gopro.domain.enum import GoProIntEnum


class StatusId(GoProIntEnum):
    """Status ID to identify statuses sent to GoProUUID.CQ_QUERY or received from GoProUUID.CQ_QUERY_RESP."""

    BATTERY_PRESENT = 1
    INTERNAL_BATTERY_BARS = 2
    OVERHEATING = 6
    BUSY = 8
    QUICK_CAPTURE = 9
    ENCODING = 10
    LCD_LOCK = 11
    VIDEO_ENCODING_DURATION = 13
    WIRELESS_CONNECTIONS_ENABLED = 17
    PAIRING_STATE = 19
    LAST_PAIRING_TYPE = 20
    LAST_PAIRING_SUCCESS = 21
    WIFI_SCAN_STATE = 22
    LAST_WIFI_SCAN_SUCCESS = 23
    WIFI_PROVISIONING_STATE = 24
    REMOTE_VERSION = 26
    REMOTE_CONNECTED = 27
    CONNECTED_WIFI_SSID = 29
    ACCESS_POINT_SSID = 30
    CONNECTED_DEVICES = 31
    PREVIEW_STREAM = 32
    PRIMARY_STORAGE = 33
    REMAINING_PHOTOS = 34
    REMAINING_VIDEO_TIME = 35
    PHOTOS = 38
    VIDEOS = 39
    OTA = 41
    PENDING_FW_UPDATE_CANCEL = 42
    LOCATE = 45
    TIMELAPSE_INTERVAL_COUNTDOWN = 49
    SD_CARD_REMAINING = 54
    PREVIEW_STREAM_AVAILABLE = 55
    WIFI_BARS = 56
    ACTIVE_HILIGHTS = 58
    TIME_SINCE_LAST_HILIGHT = 59
    MINIMUM_STATUS_POLL_PERIOD = 60
    LIVEVIEW_EXPOSURE_SELECT_MODE = 65
    LIVEVIEW_Y = 66
    LIVEVIEW_X = 67
    GPS_LOCK = 68
    AP_MODE = 69
    INTERNAL_BATTERY_PERCENTAGE = 70
    MICROPHONE_ACCESSORY = 74
    ZOOM_LEVEL = 75
    WIRELESS_BAND = 76
    ZOOM_AVAILABLE = 77
    MOBILE_FRIENDLY = 78
    FTU = 79
    NUM_5GHZ_AVAILABLE = 81
    READY = 82
    OTA_CHARGED = 83
    COLD = 85
    ROTATION = 86
    ZOOM_WHILE_ENCODING = 88
    FLATMODE = 89
    VIDEO_PRESET = 93
    PHOTO_PRESET = 94
    TIMELAPSE_PRESET = 95
    PRESET_GROUP = 96
    PRESET = 97
    PRESET_MODIFIED = 98
    REMAINING_LIVE_BURSTS = 99
    LIVE_BURSTS = 100
    CAPTURE_DELAY_ACTIVE = 101
    MEDIA_MOD_STATE = 102
    TIME_WARP_SPEED = 103
    LINUX_CORE = 104
    LENS_TYPE = 105
    HINDSIGHT = 106
    SCHEDULED_CAPTURE_PRESET_ID = 107
    SCHEDULED_CAPTURE = 108
    DISPLAY_MOD_STATUS = 110
    SD_CARD_WRITE_SPEED_ERROR = 111
    SD_CARD_ERRORS = 112
    TURBO_TRANSFER = 113
    CAMERA_CONTROL_ID = 114
    USB_CONNECTED = 115
    USB_CONTROLLED = 116
    SD_CARD_CAPACITY = 117
    PHOTO_INTERVAL_CAPTURE_COUNT = 118


class InternalBatteryBars(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#internal-battery-bars-2)"""

    ZERO = 0
    ONE = 1
    TWO = 2
    THREE = 3
    CHARGING = 4


class PairingState(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#pairing-state-19)"""

    NEVER_STARTED = 0
    STARTED = 1
    ABORTED = 2
    CANCELLED = 3
    COMPLETED = 4


class LastPairingType(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#last-pairing-type-20)"""

    NOT_PAIRING = 0
    PAIRING_APP = 1
    PAIRING_REMOTE_CONTROL = 2
    PAIRING_BLUETOOTH_DEVICE = 3


class WifiScanState(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wifi-scan-state-22)"""

    NEVER_STARTED = 0
    STARTED = 1
    ABORTED = 2
    CANCELED = 3
    COMPLETED = 4


class WifiProvisioningState(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wifi-provisioning-state-24)"""

    NEVER_STARTED = 0
    STARTED = 1
    ABORTED = 2
    CANCELED = 3
    COMPLETED = 4


class PrimaryStorage(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#primary-storage-33)"""

    UNKNOWN = -1
    OK = 0
    SD_CARD_FULL = 1
    SD_CARD_REMOVED = 2
    SD_CARD_FORMAT_ERROR = 3
    SD_CARD_BUSY = 4
    SD_CARD_SWAPPED = 8


class Ota(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#ota-41)"""

    IDLE = 0
    DOWNLOADING = 1
    VERIFYING = 2
    DOWNLOAD_FAILED = 3
    VERIFY_FAILED = 4
    READY = 5
    GOPRO_APP_DOWNLOADING = 6
    GOPRO_APP_VERIFYING = 7
    GOPRO_APP_DOWNLOAD_FAILED = 8
    GOPRO_APP_VERIFY_FAILED = 9
    GOPRO_APP_READY = 10


class LiveviewExposureSelectMode(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#liveview-exposure-select-mode-65)"""

    DISABLED = 0
    AUTO = 1
    ISO_LOCK = 2
    HEMISPHERE = 3


class MicrophoneAccessory(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#microphone-accessory-74)"""

    ACCESSORY_NOT_CONNECTED = 0
    ACCESSORY_CONNECTED = 1
    ACCESSORY_CONNECTED_AND_A_MICROPHONE_IS_PLUGGED_INTO_THE_ACCESSORY = 2


class WirelessBand(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#wireless-band-76)"""

    NUM_2_4_GHZ = 0
    NUM_5_GHZ = 1


class Rotation(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#rotation-86)"""

    NUM_0_DEGREES_UPRIGHT_ = 0
    NUM_180_DEGREES_UPSIDE_DOWN_ = 1
    NUM_90_DEGREES_LAYING_ON_RIGHT_SIDE_ = 2
    NUM_270_DEGREES_LAYING_ON_LEFT_SIDE_ = 3


class MediaModState(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#media-mod-state-102)"""

    MICROPHONE_REMOVED = 0
    MICROPHONE_ONLY = 2
    MICROPHONE_WITH_EXTERNAL_MICROPHONE = 3


class TimeWarpSpeed(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#time-warp-speed-103)"""

    NUM_15X = 0
    NUM_30X = 1
    NUM_60X = 2
    NUM_150X = 3
    NUM_300X = 4
    NUM_900X = 5
    NUM_1800X = 6
    NUM_2X = 7
    NUM_5X = 8
    NUM_10X = 9
    AUTO = 10
    NUM_1X_REALTIME_ = 11
    NUM_1_2X_SLOW_MOTION_ = 12


class LensType(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#lens-type-105)"""

    DEFAULT = 0
    MAX_LENS = 1
    MAX_LENS_2_0 = 2
    MAX_LENS_2_5 = 3
    MACRO_LENS = 4
    ANAMORPHIC_LENS = 5
    NEUTRAL_DENSITY_4 = 6
    NEUTRAL_DENSITY_8 = 7
    NEUTRAL_DENSITY_16 = 8
    NEUTRAL_DENSITY_32 = 9


class DisplayModStatus(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#display-mod-status-110)"""

    NUM_000_DISPLAY_MOD_0_HDMI_0_DISPLAY_MOD_CONNECTED_FALSE = 0
    NUM_001_DISPLAY_MOD_0_HDMI_0_DISPLAY_MOD_CONNECTED_TRUE = 1
    NUM_010_DISPLAY_MOD_0_HDMI_1_DISPLAY_MOD_CONNECTED_FALSE = 2
    NUM_011_DISPLAY_MOD_0_HDMI_1_DISPLAY_MOD_CONNECTED_TRUE = 3
    NUM_100_DISPLAY_MOD_1_HDMI_0_DISPLAY_MOD_CONNECTED_FALSE = 4
    NUM_101_DISPLAY_MOD_1_HDMI_0_DISPLAY_MOD_CONNECTED_TRUE = 5
    NUM_110_DISPLAY_MOD_1_HDMI_1_DISPLAY_MOD_CONNECTED_FALSE = 6
    NUM_111_DISPLAY_MOD_1_HDMI_1_DISPLAY_MOD_CONNECTED_TRUE = 7


class CameraControlId(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#camera-control-id-114)"""

    CAMERA_IDLE_NO_ONE_IS_ATTEMPTING_TO_CHANGE_CAMERA_SETTINGS = 0
    CAMERA_CONTROL_CAMERA_IS_IN_A_MENU_OR_CHANGING_SETTINGS_TO_INTERVENE_APP_MUST_REQUEST_CONTROL = 1
    CAMERA_EXTERNAL_CONTROL_AN_OUTSIDE_ENTITY_APP_HAS_CONTROL_AND_IS_IN_A_MENU_OR_MODIFYING_SETTINGS = 2


class UsbControlled(GoProIntEnum):
    """See [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/statuses.html#usb-controlled-116)"""

    DISABLED = 0
    ENABLED = 1
